package controles;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.FileInputStream;
import java.util.Properties;

/**
 * @autor Simon Mahieux et Jerome Baudoux
 */
public class ClavierManager extends KeyAdapter 
{
	private static ClavierManager clavier = new ClavierManager();

	// NUMERO DES TOUCHES
	private int numGauche;
	private int numDroite;
	private int numHaut;
	private int numBas;
	private int numTir;
	private int numEntrer;
	private int numPause;
	private int numEsc;	
	private int numTilt;
	
	// ETATS DES TOUCHES
	private int gauche;
	private int droite;
	private int haut;
	private int bas;
	private int tir;
	private int entrer;
	private int pause;
	private int esc;
	private int tilt;
	
	private int derniereToucheAppuye;

	/**
	 * Constructeur d'un clavier
	 */
	private ClavierManager()
	{
		lireTouches();
	}
	

	/**
	 * GESTION DE L'APPUI D'UNE TOUCHE
	 * @param keyEvent l'vnement clavier
	 */
	public void keyPressed(KeyEvent keyEvent) 
	{
		if (keyEvent.getKeyCode() == numGauche ) 
		{
			if( droite == 1 )
				droite = 2;
			
			gauche = 1;				
		} 
		if (keyEvent.getKeyCode() == numDroite ) 
		{
			if( gauche == 1 )
				gauche = 2;
			
			droite = 1;	
		} 
		if (keyEvent.getKeyCode() == numHaut ) 
		{
			if( bas == 1 )
				bas = 2;
			
			haut = 1;	
		} 
		if (keyEvent.getKeyCode() == numBas ) 
		{
			if( haut == 1 )
				haut = 2;
			
			bas = 1;	
		} 
		if (keyEvent.getKeyCode() == numTir ) 
		{
			tir = 1;
		} 
		if (keyEvent.getKeyCode() == numEntrer ) 
		{
			entrer = 1;
		}
		if (keyEvent.getKeyCode() == numPause ) 
		{
			pause = 1;
		}
		if (keyEvent.getKeyCode() == numEsc ) 
		{
			esc = 1;
		}		
		if (keyEvent.getKeyCode() == numTilt ) 
		{
			tilt = 1;
		} 
		
		derniereToucheAppuye = keyEvent.getKeyCode();
	}

	/**
	 * GESTION DES TOUCHES RELACHEES
	 * @param keyEvent l'vnement clavier
	 */
	public void keyReleased(KeyEvent keyEvent) 
	{
		if (keyEvent.getKeyCode() == numGauche ) 
		{
			if( droite == 2 )
				droite = 1;
				
			gauche = 0;
		} 
		if (keyEvent.getKeyCode() == numDroite ) 
		{
			if( gauche == 2 )
				gauche = 1;
				
			droite = 0;
		}
		if (keyEvent.getKeyCode() == numHaut ) 
		{
			if( bas == 2 )
				bas = 1;
			
			haut = 0;
		}
		if (keyEvent.getKeyCode() == numBas ) 
		{
			if( haut == 2 )
				haut = 1;
			
			bas = 0;
		}
		if (keyEvent.getKeyCode() == numTir ) 
		{
			tir = 0;
		}
		if (keyEvent.getKeyCode() == numEntrer ) 
		{
			entrer = 0;
		}
		if (keyEvent.getKeyCode() == numPause ) 
		{
			pause = 0;
		}
		if (keyEvent.getKeyCode() == numEsc ) 
		{
			pause = 0;
		}	
		if (keyEvent.getKeyCode() == numTilt ) 
		{
			tilt = 0;
		} 
	}
	
	
	/**
	 * Lire touches
	 */
	public void lireTouches()
	{
		Properties lecteur = new Properties();
		FileInputStream fichier;
		
		try
		{
			fichier = new FileInputStream("Data/clavier.sb");
			lecteur.load(fichier);
			fichier.close();
			
			numEntrer 	= 10;
			numEsc 		= 27;
			numHaut		= Integer.parseInt( lecteur.getProperty("haut"	) );	
			numBas		= Integer.parseInt( lecteur.getProperty("bas"	) );	
			numGauche	= Integer.parseInt( lecteur.getProperty("gauche") );	
			numDroite	= Integer.parseInt( lecteur.getProperty("droite") );	
			numTir		= Integer.parseInt( lecteur.getProperty("tir"	) );	
			numTilt		= Integer.parseInt( lecteur.getProperty("tilt"	) );
		}
		catch( Exception e)
		{
			numEntrer 	= 10;
			numEsc 		= 27;	
			numTilt		= 122;
			numGauche	= 37;	
			numDroite	= 39;
			numHaut		= 38;	
			numBas		= 40;	
			numTir		= 32;	
		}
	}	
	
	/**
	 * Renvoie les touches
	 * @return numeros des touches
	 */
	public int[] getListeTouches()
	{
		int[] ret = new int[6];
		ret[0] = numHaut;
		ret[1] = numBas;
		ret[2] = numGauche;
		ret[3] = numDroite;
		ret[4] = numTir;
		ret[5] = numTilt;
		return ret;
	}
	
	
	public int getDerniereToucheAppuye()
	{
		return derniereToucheAppuye;
	}
	
	public static ClavierManager getClavierManager()
	{
		return clavier;
	}

	public boolean estBas()
	{
		return bas==1;
	}

	public boolean estDroite()
	{
		return droite==1;
	}
	
	public boolean estGauche()
	{
		return gauche==1;
	}

	public boolean estHaut()
	{
		return haut==1;
	}	

	public boolean estEntrer()
	{
		return entrer==1 || tir==1;
	}
	
	public boolean estEchappe()
	{
		return esc==1;
	}	

	public boolean estTir()
	{
		return tir==1;
	}
	
	public boolean estTilt()
	{
		return tilt==1;
	}
	
	public boolean estPause()
	{
		return pause==1;
	}
	
	public void validerHaut()
	{
		haut = 0;
	}	
	
	public void validerBas()
	{
		bas = 0;
	}	
	
	public void validerEntrer()
	{
		entrer 				 = 0;
		tir    				 = 0;
		derniereToucheAppuye = 27;
	}
	
	public void validerEchappe()
	{
		esc  				 = 0;
		derniereToucheAppuye = 10;
	}	
	
	public void validerPause()
	{
		pause = 0;
	}	
	
	public void validerTir()
	{
		tir  				 = 0;
	}	
	
	public void validerTilt()
	{
		tilt  				 = 0;
	}	

	public void validerTout()
	{
		gauche 	= 0;
		droite 	= 0;
		haut 	= 0;
		bas 	= 0;
		tir 	= 0;
		entrer 	= 0;
		pause 	= 0;
		esc 	= 0;
		tilt	= 0;
	}		

}
