package controles;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Random;
import java.util.Vector;

import menus.MenuDemandeNom;
import menus.MenuPause;
import elements.Balle;
import elements.Bonus;
import elements.Dessinable;
import elements.Joueur;
import elements.Raffraichissable;
import elements.Tir;
import elements.ennemis.Ennemi;


/**
 * 
 * @author Simon Mahieux et Jerome Baudoux
 *
 */
public class Environnement implements Raffraichissable, Dessinable
{
	private static final long serialVersionUID = 1L;
	public static Environnement env;

	// MACROS	
	private static int			TEMPS_ENTRE_DEUX_TIRS_ENNEMIS = 2000;
	private static int			TEMPS_BONUS					  = 60000;
	
	public static int 			MODE_JEU 			= 0;
	public static int 			MODE_PAUSE 			= 1;
	public static int			MODE_INTRO  		= 2;
	public static int			MODE_FIN_NIVEAU 	= 3;
	public static int			MODE_GAME_OVER		= 4;
	public static int			MODE_FIN_JEU 		= 5;
	public static int			MODE_DEMANDER_NOM 	= 6;
	public static int			QUITTER				= 7;
	
	
	
	// DONNEES DU JEU
	private float 				vitesseDuJeu;
	private int 				pause;
	private long				temps;
	private long				tempsAvantProchainTirEnnemi;
	private int					score;
	private int					nombreVies;
	private int					nombreContinues;
	
	
	// BONUS
	private long				tempsBonusAugmenteTaille;
	private long				tempsBonusDiminueTaille;
	private long				tempsBonusDiminueVitesse;
	private long				tempsBonusAugmenteVitesse;
	
	
	// ELEMENTS
	private Joueur				joueur;
	private Balle				balle;
	private Vector<Tir>			tirsJoueur;
	private Vector<Tir>			tirsEnnemis;
	private Vector<Bonus>		bonus;
	private MenuPause			menuPause;
	private MenuDemandeNom		menuDemanderNom;

	/**
	 * Initialise les variables d'environnement
	 */
	private Environnement()
	{
		// VITESSE DU JEU ( VARIE SUIVANT LES OBJET )
		vitesseDuJeu 				= 1;
		tempsAvantProchainTirEnnemi = TEMPS_ENTRE_DEUX_TIRS_ENNEMIS;
		
		// VIES ET CONTINUES
		nombreVies 					= 3;
		lireOptions();
		
		// SCORE
		score						= 0;
		
		// INITIALISATION
		init();
		
		// CREATION DE LA BALLE ET DU JOUEUR ET DES AUTRES ELEMENTS
		joueur 						= new Joueur();
		balle  						= new Balle();
		tirsJoueur					= new Vector<Tir>(0);
		tirsEnnemis					= new Vector<Tir>(0);
		bonus						= new Vector<Bonus>(0);
		menuPause					= new MenuPause();
		
		// PASSE EN MODE INTRO
		setMode(MODE_INTRO);
	}
	
	/**
	 * Initialise l'environnement aux valeurs d'originine
	 */
	public static void reset()
	{
		env = new Environnement();
	}
	
	/**
	 * Renvoir l'environnement
	 * @return environnement
	 */
	public static Environnement getEnvironnement()
	{
		return env;
	}
	
	/**
	 * Initialise l'environnement
	 */
	public void init()
	{
		tempsBonusAugmenteTaille	= 0;
		tempsBonusDiminueTaille		= 0;
		tempsBonusDiminueVitesse	= 0;
		tempsBonusAugmenteVitesse	= 0;
		
		// Reinit le joeur
		if( joueur != null)
			joueur.init();
	}
	

	/**
	 * Lit le fichier options.dat
	 */
	public void lireOptions()
	{
		Properties lecteur = new Properties();

		FileInputStream fichier;
		
		try
		{
			fichier = new FileInputStream("Data/options.sb");
			lecteur.load(fichier);
			fichier.close();
			
			nombreContinues 	= Integer.parseInt( lecteur.getProperty( "nombreContinues"	) );			
		}
		catch( Exception e)
		{
			nombreContinues = 3;
		}
	}	
	
	
	// --
	// GETTERS D'ELEMENTS
	// --
	
	/**
	 * Renvoie le joueur
	 * @return Joueur
	 */
	public Joueur getJoueur()
	{
		return joueur;
	}
	
	/**
	 * Renvoie la balle
	 * @return Balle
	 */
	public Balle getBalle()
	{
		return balle;
	}	
	
	
	/**
	 * Renvoie le vecteur de tirsJoueur
	 * @return Tirs Joueurs
	 */
	public Vector<Tir> getTirsJoueur()
	{
		return tirsJoueur;
	}	
	
	
	/**
	 * Renvoie le vecteur de tirsEnnemis
	 * @return Tirs ennemis
	 */
	public Vector<Tir> getTirsEnnemis()
	{
		return tirsEnnemis;
	}	
	
	
	/**
	 * Renvoie le vecteur de Bonus
	 * @return Bonus
	 */
	public Vector<Bonus> getBonus()
	{
		return bonus;
	}	
	
	
	// --
	// GETTERS D'ETAT
	// --
	
	
	/**
	 * Renvoie vrai si le jeu n'est pas en pause 
	 * @return vrai si le jeu n'est pas en pause 
	 */
	public boolean estJeu()
	{
		return pause == MODE_JEU;
	}
	
	/**
	 * Renvoie la vitesse du jeu
	 * @return
	 */
	public float getVitesseDuJeu()
	{
		return vitesseDuJeu;
	}
	
	/**
	 * Renvoie le score
	 * @return score
	 */
	public int getScore()
	{
		return score;
	}
	
	/**
	 * Renvoie vrai si fin du jeu
	 * @return vrai si fin du jeu
	 */
	public boolean finJeu()
	{
		return pause == QUITTER;
	}
	
	

	
	
	
	// --
	// SETTERS D'ETAT
	// --
	
	/**
	 * Renvoie l'etat de la pause
	 * @return
	 */
	public void setMode( int etat )
	{
		if( etat == MODE_FIN_NIVEAU )
		{
			temps = 2500;
			SonManager.getSonManager().stopMusiqueJeu();		
			SonManager.getSonManager().jouerMusiqueFinNiveau();
			
			//PLACE LA BALLE SUR LE JOUEUR
			Environnement.getEnvironnement().getBalle().setEtat( Balle.ACCROCHEE_AU_JOUEUR );	
		}
		else if( etat == MODE_INTRO )
		{
			temps = 2300;
			SonManager.getSonManager().jouerMusiqueIntro();
		}
		else if( etat == MODE_GAME_OVER )
		{
			temps = 7000;
			SonManager.getSonManager().stopMusiqueJeu();		
			SonManager.getSonManager().jouerMusiqueGameOver();
		}
		else if( etat == MODE_FIN_JEU )
		{
			temps = 3000;
			SonManager.getSonManager().stopMusiqueJeu();	
			SonManager.getSonManager().jouerMusiqueFinJeu();	
		}
		else if( etat == MODE_JEU )
		{
			SonManager.getSonManager().jouerMusiqueJeu();
		}
		else if( etat == MODE_DEMANDER_NOM )
		{
			menuDemanderNom = new MenuDemandeNom();
		}
		
		pause = etat;
	}
	
	
	/**
	 * Gestion des bonus
	 * @param bonus type de bonus
	 */
	public void setBonus( int bonus )
	{
		// SI BONUS AUGMENTE TAILLE
		if( bonus == Bonus.AUGMENTE_TAILLE )
		{
			// SI BONUS OU NON BONUS ON MET LE TEMPS A 120
			if( tempsBonusAugmenteTaille >= 0 && tempsBonusDiminueTaille == 0 )
				tempsBonusAugmenteTaille = TEMPS_BONUS;
			// SI MALUS EN ENLEVE LE MALUS
			else if( tempsBonusAugmenteTaille == 0 && tempsBonusDiminueTaille > 0 )
				tempsBonusDiminueTaille = 0;
		}
		// SI BONUS DIMINUE  TAILLE
		else if( bonus == Bonus.DIMINUE_TAILLE )
		{
			// SI NON BONUS ON LE MET A 120
			if( tempsBonusDiminueTaille >= 0 && tempsBonusAugmenteTaille == 0 )
				tempsBonusDiminueTaille = TEMPS_BONUS;
			// SI BONUS ON L'ENLEVE
			else if( tempsBonusDiminueTaille == 0 && tempsBonusAugmenteTaille > 0 )
				tempsBonusAugmenteTaille = 0;			
		}
		
		
		
		// SI BONUS AUGMENTE VITESSE
		if( bonus == Bonus.AUGMENTE_VITESSE )
		{
			if( tempsBonusAugmenteVitesse >= 0 && tempsBonusDiminueVitesse == 0 )
				tempsBonusAugmenteVitesse = TEMPS_BONUS;
			else if( tempsBonusAugmenteVitesse == 0 && tempsBonusDiminueVitesse > 0 )
				tempsBonusDiminueVitesse = 0;
		}
		// SI BONUS DIMINUE  VITESSE
		else if( bonus == Bonus.DIMINUE_VITESSE )
		{
			if( tempsBonusDiminueVitesse >= 0 && tempsBonusAugmenteVitesse == 0 )
				tempsBonusDiminueVitesse = TEMPS_BONUS;
			else if( tempsBonusDiminueVitesse == 0 && tempsBonusAugmenteVitesse > 0 )
				tempsBonusAugmenteVitesse = 0;			
		}
		
		
		// SI BONUS AJOUTE VIE
		else if( bonus == Bonus.AJOUTE_VIE )
		{
			ajouterVie();		
		}
		// SI BONUS RETIRE VIE		
		else if( bonus == Bonus.RETIRE_VIE )
		{
			perdreVie();		
		}
	}
	
	
	/**
	 * Renvoie la vitesse du jeu
	 * @return
	 */
	public void setVitesseDuJeu( float valeurVitesse )
	{
		vitesseDuJeu = valeurVitesse;
	}	
	

	// FONCTIONS

	/**
	 *  perd une vie et rinitialise les bombes
	 */
	public void perdreVie()
	{
		// INITIALISE LA BALLE
		balle.setEtat( Balle.ACCROCHEE_AU_JOUEUR );
		
		// INITIALISE RAQUETTE JOUEUR
		joueur.init();
		
		// INITIALISE L'ENVIRONNEMENT
		init();
		
		// JOUE SON
		SonManager.getSonManager().jouerMortBalle();
		
		// DECREMENTE LE NOMBRE DE VIES
		nombreVies--;

		// SI PLUS ASSEZ DE VIE ON DECREMENTE LE NOMBRE DE CONTINUES
		if( nombreVies < 1 )
		{
			nombreVies 		= 3;
			nombreContinues--;
		}

		// SI PLUS ASSEZ DE CONTINUES ON DECLARE LE JEU FINI
		if( nombreContinues < 0)
		{
			nombreVies 		= 0;
			nombreContinues = 0;
			setMode(MODE_GAME_OVER);
		}	
	}
	
	/**
	 * Ajoute une vie
	 */
	public void ajouterVie()
	{
		// INCREMENTE LES VIES
		nombreVies++;
		
		// INCREMENTE LES CONTINUES SI PLUS DE VIE QUE PEUT EN CONTENIR UN CONTINUE
		if( nombreVies > 2 )
		{
			nombreContinues++;
			nombreVies = 1;
		}
	}
	
	
	
	/**
	 * Ajoute des points au score
	 * @param points points a ajouter
	 */
	public void ajouerPoints( int points )
	{
		score+=points;
	}
	
	
	/**
	 * Vide les vecteurs de tirs et rinitialise le temps avant le prochain tir
	 */
	public void viderVecteurs()
	{
		tirsEnnemis 				= new Vector<Tir>(0);
		tirsJoueur  				= new Vector<Tir>(0);
		bonus						= new Vector<Bonus>(0);
		tempsAvantProchainTirEnnemi = TEMPS_ENTRE_DEUX_TIRS_ENNEMIS;
	}
	
	
	/**
	 * Gestion des bonus
	 * @param temps temps de la derniere frame
	 */
	private void gererBonus( long temps )
	{
		// SI BONUS AUGMENTE TAILLE
		if( tempsBonusAugmenteTaille > 0 )
		{
			tempsBonusAugmenteTaille -= temps;
			joueur.setTaille( Joueur.TAILLE_GRANDE );
		}
		else
		{
			// SI BONUS AUGMENTE TAILLE
			if( tempsBonusDiminueTaille > 0 )
			{
				tempsBonusDiminueTaille -= temps;
				joueur.setTaille( Joueur.TAILLE_PETITE );
			}
			else
			{
				joueur.setTaille( Joueur.TAILLE_NORMALE );
			}
		}
		
		// SI BONUS AUGMENTE TAILLE
		if( tempsBonusDiminueVitesse > 0 )
		{
			tempsBonusDiminueVitesse -= temps;
			vitesseDuJeu = .6f;
		}
		else
		{
			// SI BONUS AUGMENTE TAILLE
			if( tempsBonusAugmenteVitesse > 0 )
			{
				tempsBonusAugmenteVitesse -= temps;
				vitesseDuJeu = 1.4f;
			}
			else
			{
				vitesseDuJeu = 1;
			}
		}
		

	}
	
	
	//TEMPS_ENTRE_DEUX_TIRS_ENNEMIS
	/**
	 * Gere les tirs ennemis
	 * @param t temps de la derniere frame
	 */
	private void gererTirsEnnemis( long temps )
	{
		// Diminue le temps
		if( tempsAvantProchainTirEnnemi > 0 )
			tempsAvantProchainTirEnnemi -= temps;
		
		// SI ON DOIS TIRER
		else if( ! NiveauManager.getNiveauManager().levelClear() )
		{
			// CHOISI UN ENNEMI AU HAZARD
			Ennemi e = NiveauManager.getNiveauManager().getEnnemis().get( (int)(new Random().nextFloat()*NiveauManager.getNiveauManager().getEnnemis().size()) );
			
			// SI IL PEUT TIRER ALORS LE FAIT TIRER
			if( e.pretaTirer() )
			{
				e.tirer();
				tempsAvantProchainTirEnnemi = TEMPS_ENTRE_DEUX_TIRS_ENNEMIS;
			}
		}
	}
	
	
	/**
	 * Raffraichir l'environnement
	 * @param t temps
	 */
	public void raffraichir( long t )
	{

		// SI EN MODE INTRO
		if( pause == MODE_INTRO )
		{
			temps -= t;

			// QUITTE LE MODE INTRO ET COMMENCE LE JEU
			if( temps <= 0)
				setMode(MODE_JEU);
		}
		
		
		// SI EN MODE JEU
		else if( pause == MODE_JEU )
		{
			//SI LE NIVEAU EST FINI
			if( NiveauManager.getNiveauManager().levelClear() )
				setMode(MODE_FIN_NIVEAU);
			
			// GESTION DES TIRS ENNEMIS ET DES BONUS
			gererTirsEnnemis( t );
			gererBonus( t );
			
			
			// DECREMENTE LE TEMPS SI > 0 SINON BLOQUE A 0
			if( NiveauManager.getNiveauManager().getTempsRestant() > 0 )
				NiveauManager.getNiveauManager().setTempsRestant( NiveauManager.getNiveauManager().getTempsRestant() - t );
			else if( NiveauManager.getNiveauManager().getTempsRestant() < 0 )
				NiveauManager.getNiveauManager().setTempsRestant(0);
			
			
			// SI ON APPUYE SUR ECHAPE ON PASSE EN MODE PAUSE
			if( ClavierManager.getClavierManager().estEchappe() )
			{
				SonManager.getSonManager().pauseMusiqueJeu();
				ClavierManager.getClavierManager().validerEchappe();
				setMode(MODE_PAUSE);
			}
			
		}
		
		
		// SI EN MODE PAUSE
		else if( pause == MODE_PAUSE )
		{
			menuPause.raffraichir(t);
			
			if( menuPause.continuer() )
			{
				SonManager.getSonManager().repriseMusiqueJeu();
				ClavierManager.getClavierManager().validerTout();
				setMode(MODE_JEU);
			}

			else if( menuPause.retourAuMenu() )
			{
				setMode(MODE_GAME_OVER);
				ClavierManager.getClavierManager().validerTout();
			}

		}
		
		
		// SI EN MODE NIVEAU SUIVANT
		else if( pause == MODE_FIN_NIVEAU )
		{
			temps -= t;
			
			// QUITTE LE MODE FIN_NIVEAU ET COMMENCE LE NIVEAU SUIVANT
			if( temps <= 0)
				NiveauManager.getNiveauManager().niveauSuivant();
		}
		
		
		// SI EN MODE NIVEAU FIN JEU
		else if( pause == MODE_FIN_JEU )
		{
			temps -= t;
			
			// QUITTE LE MODE FIN_JEU ET DEMANDE LE NOM SI MEILLEUR SCORE
			if( temps <= 0)
				setMode(MODE_DEMANDER_NOM);
		}
		
		
		// SI EN MODE GAME OVER
		else if( pause == MODE_GAME_OVER )
		{
			temps -= t;
			
			if( temps <= 0 )
				setMode(MODE_DEMANDER_NOM);
		}
		
		// SI EN MODE GAME OVER
		else if( pause == MODE_DEMANDER_NOM )
		{
			menuDemanderNom.raffraichir(t);
			
			if( menuDemanderNom.menuPrincipal() )
				pause = QUITTER;
		}
	}

	
	
	/**
	 * Dessine les informations de l'environnement
	 */
	public void dessiner(Graphics2D graphics2D) 
	{
		// DESSINE LE TABLEAU DE JEU
		graphics2D.drawImage( ImageManager.getImagemanager().getImageTableauJeu().getImage(), 0, 0, null);

		// DESSINE LE SCORE
		graphics2D.setColor  ( Color.black );
		graphics2D.drawString( Integer.toString( score ) ,95,18);
		
		// DESSINE LE NOM DU NIVEAU
		graphics2D.drawString("NIVEAU "+NiveauManager.getNiveauManager().getNumNiveau(),570,18);
		
		// DESSINE LE NOMBRE DE VIES
		for( int i=0; i<nombreVies; i++ )
			graphics2D.drawImage( ImageManager.getImagemanager().getImageBalle().getImage(), 750 + i*25, 7, null);
		
		// DESSINE LE NOMBRE DE CONTINUES
		graphics2D.drawString( Integer.toString( nombreContinues ) ,945,18);	
		
		// DESSINE LES BONUS
		// TAILLE
		if( tempsBonusAugmenteTaille > 0 )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageBonus( Bonus.AUGMENTE_TAILLE - 1 ).getImage(), 180, 5, null);
			graphics2D.setColor  ( Color.blue );
			graphics2D.drawString( Integer.toString( (int)(tempsBonusAugmenteTaille/1000) ) ,210,18);
		}
		else if( tempsBonusDiminueTaille > 0 )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageBonus( Bonus.DIMINUE_TAILLE - 1 ).getImage(), 180, 5, null);
			graphics2D.setColor  ( Color.red );
			graphics2D.drawString( Integer.toString( (int)(tempsBonusDiminueTaille/1000) ) ,210,18);
		}
	
		// VITESSE
		if( tempsBonusAugmenteVitesse > 0 )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageBonus( Bonus.AUGMENTE_VITESSE - 1 ).getImage(), 280, 5, null);
			graphics2D.setColor  ( Color.red );
			graphics2D.drawString( Integer.toString( (int)(tempsBonusAugmenteVitesse/1000) ) ,310,18);
		}
		else if( tempsBonusDiminueVitesse > 0 )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageBonus( Bonus.DIMINUE_VITESSE - 1 ).getImage(), 280, 5, null);
			graphics2D.setColor  ( Color.blue );
			graphics2D.drawString( Integer.toString( (int)(tempsBonusDiminueVitesse/1000) ) ,310,18);
		}
		

		// DESSINE LE TILT
		graphics2D.drawString( Integer.toString( (int)NiveauManager.getNiveauManager().getNumTilt() ) ,290,18);		
		
		// DESSINE LE TEMPS
		graphics2D.setColor  ( Color.black );
		graphics2D.setFont(new Font(Font.MONOSPACED, Font.BOLD, 18));	
		graphics2D.drawString( Integer.toString( (int)NiveauManager.getNiveauManager().getTempsRestant()/1000 ) ,485,20);			
	
		
		// SI EN MODE INTRO
		if( pause == MODE_INTRO )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageInfos(0).getImage(), 362, 284, null );
		}
		
		// SI EN MODE FIN NIVEAU
		else if( pause == MODE_FIN_NIVEAU )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageInfos(1).getImage(), 362, 284, null );
		}
		
		// SI EN MODE GAME OVER
		else if( pause == MODE_GAME_OVER )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageInfos(2).getImage(), 362, 284, null );
		}
		
		// SI EN MODE FIN JEU
		else if( pause == MODE_FIN_JEU )
		{
			graphics2D.drawImage( ImageManager.getImagemanager().getImageInfos(4).getImage(), 362, 284, null );
		}
		
		// SI EN MODE DEMANDER NOM
		else if( pause == MODE_DEMANDER_NOM )
		{
			menuDemanderNom.dessiner( graphics2D );
		}
		
		// SI EN MODE PAUSE
		else if( pause == MODE_PAUSE )
		{
			menuPause.dessiner(graphics2D);
		}
		
		
	}
	
}
