package controles;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;
import elements.ennemis.*;

public class NiveauManager 
{
	private static NiveauManager niveauManager;
	
	// MACROS
	public static int NOMBRE_D_ENNEMIS_PAR_LIGNE  = 24;
	public static int NOMBRE_D_ENNEMIS_PAR_COLONE = 29;
	
	private int					nombreNiveaux;
	private int					numNiveau;	
	private int					nbBriquesInvincibles;
	private long				tempsRestant;
	private int					nbTilt;	
		
	private Vector<Ennemi>		ennemis;

	
	
	
	/**
	 * Gestionnaire de niveau
	 */
	private NiveauManager()
	{
		// LIS LE NOMBRE DE NIVEAUX
		File file = new File("Niveaux/");
		nombreNiveaux = file.list().length;

		// INITIALISE LE PREMIER NIVEAU
		numNiveau = 0;
		niveauSuivant();
		
		nbTilt = 3;
	}
	private NiveauManager( int niveau )
	{
		// INITIALISE LE PREMIER NIVEAU
		nombreNiveaux = niveau+1;
		numNiveau = niveau;
		niveauSuivant();
		
		nbTilt = 3;
	}
	
	
	/**
	 * Retourne le Gestionnaire de Niveau
	 * @return Gestionnaire de Niveau
	 */
	public static NiveauManager getNiveauManager()
	{
		return niveauManager;
	}
	
	/**
	 * Initialise l'environnement aux valeurs d'originine
	 */
	public static void init()
	{
		niveauManager = new NiveauManager();
	}
	public static void init( int niveau )
	{
		niveauManager = new NiveauManager( niveau );
	}
	
	
	/**
	 * Renvoie le temps restant
	 * @return temps restant
	 */
	public long getTempsRestant()
	{
		return tempsRestant;
	}
	
	/**
	 * Modifie la valeur du temps Restant
	 * @param temps temps
	 */
	public void setTempsRestant( long temps )
	{
		tempsRestant = temps;
	}
	
	
	// FONCTIONS
	
	/**
	 * Charge le niveau suivant
	 */
	public void niveauSuivant()
	{
		// DONNE DES POINTS SI LE TEMPS EST > 0
		if( tempsRestant > 0 && tempsRestant < 98940 )
			Environnement.getEnvironnement().ajouerPoints( (int)(tempsRestant/1000) * 1 + 50 );
		
		// SI PAS FAIT TOUS LES NIVEAUX
		if( numNiveau < nombreNiveaux )
		{
			Environnement.getEnvironnement().setMode( Environnement.MODE_INTRO );
			
			// INCREMENTE LE NIVEAU
			numNiveau++;
			
			// CREE LE TABLEAU D'ENNEMIS
			ennemis = new Vector<Ennemi>(0);
			
			// REMISE DU TEMPS A 99s
			tempsRestant = 99000;
			
			// INITIALISE L'ENVIRONNEMENT POUR LE CHANGEMENT DE NIVEAU
			Environnement.getEnvironnement().viderVecteurs();
			Environnement.getEnvironnement().init();
			
			// REMPLI LE TABLEAU AVEC LES ENNEMIS DU NIVEAU 1
			chargerFichier( "niveau"+numNiveau+".sb" );
		}
		// SINON
		else
		{
			Environnement.getEnvironnement().setMode( Environnement.MODE_FIN_JEU );			
		}
	}
	
	/**
	 * Charge un fichier dans la liste des ennemis
	 * @param file nom du fichier d'entree
	 */
	private void chargerFichier( String file )
	{
		// INIT
		nbBriquesInvincibles = 0;
		BufferedReader fichier ;
		nbTilt = 3;
		try
		{
			fichier = new BufferedReader( new FileReader("Niveaux/"+file) );
			String ligne;
			int j = 0;
			
			// DEFINI L'IMAGE
			ImageManager.getImagemanager().setImageNiveau( fichier.readLine() );
			
			// AJOUT DES ENNEMIS
			while( j < NOMBRE_D_ENNEMIS_PAR_COLONE )
			{
				// LIS LA LIGNE
				ligne 				= fichier.readLine();
				String[] donnee 	= ligne.split(" ");
				
				// SI LA LIGNE N'EST PAS VIDE ET NE COMMENCE PAS PAR #
				if( ligne.length() > 0 && ! ligne.substring(0, 1).equals("#") )
				{
					// TRAITE LA LIGNE
					for( int i=0; i< donnee.length; i++ )
					{
						String[] valeur = donnee[i].split(":");
						
						// AJOUTE LES ENNEMIS
						switch( Integer.parseInt(valeur[0]) )
						{
							case 1:
								
								ennemis.add( new EnnemiBasique(20+i*41,21*j + 35, 1, Integer.parseInt( valeur[1] ) ) );
								break;
							case 2:
							case 3:
							case 4:	
								ennemis.add( new Ennemi(20+i*41,21*j + 35, Integer.parseInt( valeur[0] ),Integer.parseInt( valeur[0] )-1, Integer.parseInt( valeur[1] ) ) );
								break;
							case 5:
								ennemis.add( new EnnemiInvincibleTir(20+i*41,21*j + 35, 1, Integer.parseInt( valeur[1] ) ) );							
								break;
							case 6:
								ennemis.add( new EnnemiInvincibleBalle(20+i*41,21*j + 35, 1, Integer.parseInt( valeur[1] ) ) );	
								break;
							case 7:
								ennemis.add( new EnnemiInvincibleTirReponds(20+i*41,21*j + 35, 1, Integer.parseInt( valeur[1] ) ) );							
								break;							
							case 8:
								ennemis.add( new EnnemiInvincible(20+i*41,21*j + 35, Integer.parseInt( valeur[1] ) ) );	
								nbBriquesInvincibles++;
								break;
							case 9:
								ennemis.add( new EnnemiFantome(20+i*41,21*j + 35,1, Integer.parseInt( valeur[1] ) ) );	
								break;								
						}
					}
					
					j++;
				}
			}
		}
		catch( Exception e)
		{
			if( e.getMessage()!= null)
				System.err.println(e.getMessage());
		}
		
		
	}
	
	
	/**
	 * Renvoie vrai si le niveau est termin�
	 * @return vrai si le niveau est termin�
	 */
	public boolean levelClear()
	{
		return ennemis.size() <= nbBriquesInvincibles;
	}

	
	
	
	// GETTERS
	
	
	/**
	 * Renvoie le num�ro du niveau
	 * @return num�ro du niveau
	 */
	public int getNumNiveau()
	{
		return numNiveau;
	}	
	
	/**
	 * Renvoie le num�ro du niveau
	 * @return num�ro du niveau
	 */
	public int getNumTilt()
	{
		return nbTilt;
	}		
	
	/**
	 * Renvoie la liste des ennemis
	 * @return Ennemis
	 */
	public Vector<Ennemi> getEnnemis()
	{
		return ennemis;
	}
	
	/**
	 * Renvoie un Ennemi
	 * @param i numero de l'Ennemi
	 * @return Ennemi
	 */
	public Ennemi getEnnemi( int i )
	{
		return ennemis.get(i);
	}
	
	
	/**
	 * Defini le nombre de niveaux
	 * @param i nombre niveaux
	 */
	public void setNombreNiveaux( int i )
	{
		nombreNiveaux = i;
	}

	/**
	 * Defini le nombre de niveaux
	 * @param i nombre niveaux
	 */
	public void setNombreTilt( int i )
	{
		nbTilt = i;
	}
}
