package controles;

import java.awt.DisplayMode;
import java.awt.Frame;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;

/**
 * 
 * @author Simon Mahieux et Jerome Baudoux
 *
 */
public class ScreenManager 
{

	private boolean isFullScreen;
    private GraphicsDevice gd;

    /**
     * Constructeur par defaut.
     */
    public ScreenManager() 
    {
    	isFullScreen = false;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        gd = ge.getDefaultScreenDevice();
    }

    /**
     * Passage de la fenetre en mode plein ecran.
     * 
     * @param window La fenetre
     * @param displayMode mode graphique
     */
    public void setFullScreen(Frame window, DisplayMode displayMode) 
    {
        if( window != null )
        {
        	window.dispose();

            if ( gd.isFullScreenSupported() ) 
            {
                window.setUndecorated(true);
                window.setResizable(false);
                
            	gd.setFullScreenWindow(window);
            
                if( displayMode != null && gd.isDisplayChangeSupported() ) 
                {
                    try 
                    {
                        gd.setDisplayMode(displayMode);
                        isFullScreen = true;
                    } 
                    catch ( Exception e )  { }
                }
            }
            else
            	System.err.println("Mode Plein Ecran Non Suporte");

            
            //Si on pas reussi a mettre en plein ecran on remet les bords
            if( ! isFullScreen )
            	restoreFullScreen(window);
        }
        else
        {
        	System.err.println("Erreur, Fenetre invalide");
        }
    }


    /**
     * Mode Fenetre
     */
    public void restoreFullScreen(Frame window) 
    {
    	if( isFullScreen )
    	{
	        try
	        {
	        	window.dispose();
		        gd.setFullScreenWindow(null);
	        }
	        catch( Exception e ){}
	        
	        window.setUndecorated(false);	     
    	}
        
    
        window.setVisible(true);  
		window.setLocationRelativeTo( window.getParent() );
        isFullScreen = false;
    }
    
    public boolean isFullScreen()
    {
    	return isFullScreen;
    }
}