package editeur;

import java.io.*;

/**
 * 
 * @author Simon Mahieux et Jerome Baudoux
 *
 */
public class Niveau 
{
    private String chemin;
    private Grille tab;
    private int num;
    
    /**
     * Constructeur de niveau
     * @param fichier fichier niveau
     */
    public Niveau(String fichier)
    {
        tab = new Grille();
        chemin = new String("Images/Niveaux/defaut.png");
        charge(fichier);
    }
    
    /**
     * Constructeur par defaut de niveau
     */
    public Niveau()
    {
        reset();
    }
    
    /**
     * Effacer cases
     */
    public void reset()
    {
        tab = new Grille();
        chemin = new String("Images/Niveaux/defaut.png");
        num=0;
    }
    
    /**
     * Renvoie le nombre de colone
     * @return nombre de colone
     */
    public int getNbCol()
    {
        return tab.getNbCol();
    }
    
    /**
     * Renvoie le nombre de lignes
     * @return nombre de lignes
     */
    public int getNbLig()
    {
        return tab.getNbLig();
    }
    
    /**
     * Renvoie le chemin
     * @return chemin
     */
    public String getChemin()
    {
        return chemin;
    }
    
    /**
     * Renvoie le numero du niveau
     * @return numero de niveau
     */
    public int getNiv()
    {
        return num;
    }
    
    /**
     * Definie le niveau
     * @param niveau niveau
     */
    public void setNiv(int niveau)
    {
    	num = niveau;
    }
    
    /**
     * Definie le chemin
     * @param ch chemin
     */
    public void setChemin(String ch)
    {
        chemin = ch;
    }
    
    /**
     * Modifie la case
     * @param ligne ligne
     * @param colonne colone
     * @param nCase nouvelle case
     */
    public void modCase(int ligne, int colonne, Case nCase)
    {
        tab.modCase(ligne, colonne, nCase);
    }
    
    /**
     * Renvoie case
     * @param ligne ligne
     * @param colonne colone
     * @return case
     */
    public Case getCase(int ligne, int colonne)
    {
        return tab.getCase(ligne, colonne);
    }
    
    /**
     * Charger niveau
     * @param fichier fichier niveau
     */
    public void charge(String fichier)
    {
        try
        {
            num = Integer.parseInt( fichier.substring(6, fichier.indexOf(".")) );
            BufferedReader br ;
            br = new BufferedReader( new FileReader("Niveaux/"+fichier) );
            chemin = br.readLine(); // Lit la premiere ligne : l'image de fond
            
            // Ici on lit les trois lignes du fichier (ligne vide + comm + ligne vide)
            for(int sauteligne=0;sauteligne<3;sauteligne++)
            {
                br.readLine();
            }

            // Ajout des cases
            for(int li=0;li<tab.getNbLig();li++)
            {
                String ligne;
                ligne = br.readLine();
                String[] donnee	= ligne.split(" ");

                for( int co=0; co< donnee.length; co++ )
                {
                        String[] valeur = donnee[co].split(":");
                        Case brique = new Case(Integer.parseInt(valeur[0]),Integer.parseInt(valeur[1]));
                        tab.modCase(li, co, brique);
                }
            }
            br.close();
        }
        catch( Exception e)
        {
            if( e.getMessage()!= null)
            {
                    System.err.println("[ERREUR] Chargement de fichier.");
                    e.printStackTrace();
            }
        }
    }
    
    /**
     * Enregistrer niveau
     * @param numero numero du niveau
     */
    public void enregistre(String numero)
    {
        String fichier="niveau"+new Integer(numero)+".sb";
        num = new Integer(numero);
        
        try
        {
            // Enregistre le niveau dans le fichier :)
            PrintWriter fp;
            String sep = System.getProperty("line.separator");
            fp =  new PrintWriter(new BufferedWriter(new FileWriter("Niveaux/"+fichier)));
   
            fp.print(chemin);
            fp.print(sep+sep+"# Niveau genere par l'Editeur de niveau"+sep+sep);
            fp.print(tab.toString());            
            fp.close();
        }
        catch( Exception e)
        {
            if( e.getMessage()!= null)
            {
                    System.err.println("[ERREUR] Enregistrement de fichier.");
                    e.printStackTrace();
            }
        }
    }
}
