package elements;

import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.util.Random;

import controles.ClavierManager;
import controles.Environnement;
import controles.ImageManager;
import controles.NiveauManager;
import controles.SonManager;

public class Balle extends ObjetMobile
{
	public static int LIBRE = 0;
	public static int ACCROCHEE_AU_JOUEUR = 1;
	
	private int etat;
	
	int lastX;
	int lastY;
	int lastAngle;
	
	/**
	 * Balle rebondissante
	 * @param valeurX Position en X
	 * @param valeurY Position en Y
	 * @param valeurVitesse Vitesse
	 * @param valeurAngle Angle
	 */
	public Balle( int valeurX, int valeurY, float valeurVitesse, float valeurAngle, int e)
	{
		super(valeurX,valeurY,valeurVitesse,valeurAngle);
		
		// DEFINI l4ETAT
		etat 	= e;		
		
		// DEFINI l'IAMGE
		img = ImageManager.getImagemanager().getImageBalle();
	}
	
	/**
	 * Balle rebondissante colle au joueur
	 */
	public Balle()
	{
		this(-100,-100,0.35f,-45,ACCROCHEE_AU_JOUEUR);
	}
	
	
	// GETTERS
	
	/**
	 * Recupre l'air de l'objet
	 * @return ait de l'objet par defaut un retangle de la taille de l'image
	 */
	public Area getArea()
	{
		return new Area( new Ellipse2D.Double( x, y, img.getIconWidth(), img.getIconHeight()));
	}
	
	/**
	 * Renvoie l'etat
	 * @return etat
	 */
	public int getEtat()
	{
		return etat;
	}
	
	public int getLastX()
	{
		return lastX;
	}
	
	public int getLastY()
	{
		return lastY;
	}
	
	public void setLast()
	{
		lastX = (int)x;
		lastY = (int)y;
		lastAngle = (int)angle;
	}
	
	public int getLastAngle()
	{
		return lastAngle;
	}
	
	
	
	// SETTERS
	
	public void setEtat( int e )
	{
		if( e == ACCROCHEE_AU_JOUEUR)
		{
			x = Environnement.getEnvironnement().getJoueur().getX() - getWidth()/2;
			y = Environnement.getEnvironnement().getJoueur().getY() - getHeight();
			vitesse = 0.35f;
			angle 	= -45f;
			etat 	= ACCROCHEE_AU_JOUEUR;
		}
		
		etat = e;
	}
	
	
	
	// FONCTIONS
	
	
	/**
	 * Traite la collision entre une balle et un tir
	 * @param t tir
	 */
	public void collision( Tir t )
	{
		// INTERSECTION DES AIRES
		Area aireBalle	= getArea();
		aireBalle.intersect( t.getArea() );
		
		// SI IL Y A BIEN INTERSECTION
		if( ! aireBalle.isEmpty() )
		{
			// SI LA BALLE MONTE ET QUE LE TIR DESCEND OU SI LA BALLE DESCEND ET QUE LE TIR MONTE
			if( ( getVitesseY() < 0 && t.getVitesseY() > 0 ) || ( getVitesseY() > 0 && t.getVitesseY() < 0 ) )
			{
				setVitesseXY( getVitesseX(), -getVitesseY());				
			}
			// SINON DECALE SUIVANT LE SENS
			else
			{
				if( getVitesseX() > 0 )
					setAngle( getAngle() - 10 );
				else
					setAngle( getAngle() + 10 );	
			}
			
			t.tuer();
			
			// DONNE 1 POINTS
			Environnement.getEnvironnement().ajouerPoints( 1 );
			
			//JOUE UN SOND DE REBOND
			SonManager.getSonManager().jouerRebond();
		}
	}
	
	/**
	 * Mise a jour de la position de la balle
	 */
	public void raffraichir( long temps)
	{
		//lastX  		= (int) x;
		//lastY  		= (int) y;
		//lastAngle 	= (int) angle;
		
		// SI LA BALLE A DEJA ETE LANCEE
		if( etat == LIBRE )
		{
			super.raffraichir(temps);
			
			// RECUPERE LES VITESSES EN X ET Y
			double vitesseX = getVitesseX();
			double vitesseY = getVitesseY();
			
			boolean rebond = false;
			
			// SI TOUCHE UN BORD ON EFFETUE UN CHANGEMENT DE LA VITESSE
			if( x > 1024 - 6 - img.getIconWidth() )
			{
				x = 1024 - 6 - img.getIconWidth();
				vitesseX = -vitesseX;
				rebond = true;
			}
			else if( x < 6 )
			{
				x = 6;
				vitesseX = -vitesseX;
				rebond = true;
			}
			if( y < 27 )
			{
				y = 27;
				vitesseY = -vitesseY;
				rebond = true;
			}
			else if( y > 768 )
			{

				Environnement.getEnvironnement().perdreVie();
			}
			
			// MET A JOUR L'ANGLE
			if( rebond )
			{
				SonManager.getSonManager().jouerRebond();
				setVitesseXY((float)vitesseX, (float)vitesseY);
			}
			
			// SI TILT
			if( ClavierManager.getClavierManager().estTilt() && NiveauManager.getNiveauManager().getNumTilt()>0)
			{
				NiveauManager.getNiveauManager().setNombreTilt(NiveauManager.getNiveauManager().getNumTilt()-1);
				angle = new Random().nextFloat()*360;
				
				// EVITE DES ANGLES TROP HORIZONTAUX
				if(angle> 170 && angle<190)
					angle+=10+new Random().nextFloat()*120;
				else if(angle>350 || angle <10)
					angle-=10+new Random().nextFloat()*120;
				
				ClavierManager.getClavierManager().validerTilt();				
			}
		}
		else if( etat == ACCROCHEE_AU_JOUEUR )
		{
			//REFRESH DES COORDONNEES POUR SUIVRE LA RAQUETTE
			x = Environnement.getEnvironnement().getJoueur().getX() - getWidth()/2;
			y = Environnement.getEnvironnement().getJoueur().getY() - getHeight();
		}
	}
}
