package elements;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import controles.ClavierManager;
import controles.Environnement;
import controles.ImageManager;
import controles.SonManager;

public class Joueur extends ObjetMobile
{
	// MACROS
	private static int TEMPS_ENTRE_DEUX_TIRS = 1000;
	public  static int TAILLE_PETITE		 = 5;
	public  static int TAILLE_NORMALE		 = 20;
	public  static int TAILLE_GRANDE		 = 40;

	// DONNEES
	private int 	premierDeplacement;
	private int 	tailleRaquette;
	private int 	tempsAvantProchainTir;
	
	private long 	tempsFreeze;
	private long	tempsAffichageLanceBalle;
	
	/**
	 * Raquette
	 */
	public Joueur()
	{
		super( 450,730,0,0);
		init();
	}
	
	/**
	 * Initialise le joueur
	 */
	public void init()
	{
		premierDeplacement 			= 0;
		tailleRaquette				= 20;
		tempsAvantProchainTir 		= 0;
		tempsFreeze					= 0;
		tempsAffichageLanceBalle 	= 0;		
	}
	
	
	// GETTERS
	
	/**
	 * Recupre l'air de l'objet
	 * @return ait de l'objet par defaut un retangle de la taille de l'image
	 */
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double( x - getWidth()/2, y + 7, getWidth(), getHeight() ));
	}
	
	/**
	 * Largeur de la raquette
	 * @return Largeur de la raquette
	 */
	public int getWidth()
	{
		return ImageManager.getImagemanager().getImageJoueur(0).getIconWidth() + ImageManager.getImagemanager().getImageJoueur(2).getIconWidth() + tailleRaquette;
	}
	
	/**
	 * Hauteur de la raquette
	 * @return Hauteur de la raquette
	 */
	public int getHeight()
	{
		return ImageManager.getImagemanager().getImageJoueur(0).getIconHeight();
	}
	
	
	// SETTERS
	
	public void setTaille( int taille )
	{
		tailleRaquette = taille;
	}
	
	
	// FONCTIONS
	
	/**
	 * Freeze le joueur
	 */
	private void freeze()
	{
		if( tempsFreeze <= 0 )
		{
			SonManager.getSonManager().jouerMortBalle();
			tempsFreeze = 2000;
		}
	}
	
	/**
	 * Traite la collision entre une balle et la raquette
	 * @param b
	 */
	public void collision( Balle b )
	{
		
		int[] posX;
		int[] posY;
		
		// RECUPERATION DES AIRES
		Area airBalle	= b.getArea();
		
		Area airHaut 	= new Area	( new Rectangle.Double((int)(x - getWidth()/2), y, getWidth(), getHeight()/2) );
		
		posX 			= new int[] {(int)x - getWidth()/2 ,(int)(x),(int)x - getWidth()/2};
		posY 			= new int[] {(int)y,(int)(y+getHeight()/2),(int)(y+getHeight())};
		Area airGauche 	= new Area	( new Polygon(posX, posY, 3));
	
		posX 			= new int[] {(int)x + getWidth()/2 ,(int)(x),(int)x + getWidth()/2};
		posY 			= new int[] {(int)y,(int)(y+getHeight()/2),(int)(y+getHeight())};
		Area airDroite 	= new Area	( new Polygon(posX, posY, 3));

		// INTERSECTION DES AIRES
		airGauche.intersect( airBalle );
		airDroite.intersect( airBalle );
		airHaut.intersect  ( airBalle );
		
		// AIRES ( Largeur * hauteur ) DES AREA
		double airGaucheVal = airGauche.getBounds2D().getWidth() 	* airGauche.getBounds2D().getHeight();
		double airDroiteVal = airDroite.getBounds2D().getWidth() 	* airDroite.getBounds2D().getHeight();
		double airHautVal 	= airHaut.getBounds2D().getWidth() 		* airHaut.getBounds2D().getHeight();
		
		// BOOLEEN DEFINISSANT LE REBOND
		boolean collisionGauche = false;
		boolean collisionDroite = false;
		boolean collisionHaut 	= false;
		
		// COLLISION HAUT GAUCHE
		if( (! airHaut.isEmpty() && b.getVitesseY() > 0) && (! airGauche.isEmpty() && b.getVitesseX() > 0))
		{
			if( airGaucheVal > airHautVal )
				collisionGauche = true;
			else
				collisionHaut 	= true;
		}
		// COLLISION HAUT DROITE
		else if( (! airHaut.isEmpty() && b.getVitesseY() > 0) && (! airDroite.isEmpty() && b.getVitesseX() < 0))
		{
			if( airDroiteVal > airHautVal )
				collisionDroite = true;
			else
				collisionHaut 	= true;	
		}	
		// SI COLLISION EN HAUT
		else if( ! airHaut.isEmpty() && b.getVitesseY() > 0 )
		{
			collisionHaut = true;
		}
		// SI COLLISION A GAUCHE
		else if( ! airGauche.isEmpty() && b.getVitesseX() > 0 )
		{
			collisionGauche = true;
		}
		// SI COLLISION A DROITE
		else if( ! airDroite.isEmpty() && b.getVitesseX() < 0 )
		{
			collisionDroite = true;
		}
		
		// SI IL Y A EU COLLISION
		if( collisionHaut || collisionGauche || collisionDroite)
		{
			if( collisionHaut )
			{
				b.setY( (int)y - b.getHeight() );
				b.setVitesseXY( b.getVitesseX(), - b.getVitesseY());
			}
			else if( collisionGauche )
			{
				b.setX( (int)x - getWidth()/2 - b.getWidth() );		
				b.setVitesseXY( - b.getVitesseX(), b.getVitesseY());
			}
			else if( collisionDroite )
			{
				b.setX( (int)x + getWidth()/2 + getWidth() );
				b.setVitesseXY( - b.getVitesseX(), b.getVitesseY());
			}
			
			// SI LA BALLE TOUCHE LA RAQUETTE EN MOUVEMENT SON ANGLE EST MODIFIE
			b.setAngle( b.getAngle() + getVitesseX()*10 );
	
			// JOUE LES SONS NECESSAIRES
			SonManager.getSonManager().jouerRebond();
		}
	}		

	
	
	
	/**
	 * Traite la collision entre un tir et la raquette
	 * @param t tir ennemi
	 */
	public void collision( Tir t )
	{
		// INTERSECTION DES AIRES
		Area aireRaquette	= getArea();
		aireRaquette.intersect( t.getArea() );
		
		// SI IL Y A BIEN INTERSECTION
		if( ! aireRaquette.isEmpty() )
		{
			t.tuer();
			freeze();
		}
	}
	
	
	
	/**
	 * Traite la collision entre un bonus et la raquette
	 * @param b
	 */
	public void collision( Bonus b )
	{
		// INTERSECTION DES AIRES
		Area aireRaquette	= getArea();
		aireRaquette.intersect( b.getArea() );
		
		// SI IL Y A BIEN INTERSECTION
		if( ! aireRaquette.isEmpty() )
		{
			// AJOUTE LE BONUS
			Environnement.getEnvironnement().setBonus( b.getType() );
			
			// DONNE 10 POINTS
			Environnement.getEnvironnement().ajouerPoints( 10 );
			
			// DETRUIT LE BONUS
			b.tuer();
			
			//JOUE UN SOND DE REBOND
			SonManager.getSonManager().jouerBonus();
		}
	}
	
	
	/**
	 * Gere les tirs du joueur
	 */
	private void gestionTirs( long temps )
	{
		// DIMINUE LE TEMPS AVANT LE PROCHAIN TIR DU TEMPS DE LA DERNIERE FRAME
		if( tempsAvantProchainTir > 0)
			tempsAvantProchainTir -= temps;
		
		// SI ON PEUT TIRER ET QUE LE BOUTON EST APPUYE
		if( tempsAvantProchainTir <= 0 && ClavierManager.getClavierManager().estTir() )
		{
			
			
			if( Environnement.getEnvironnement().getBalle().getEtat() == Balle.LIBRE )
			{
				tempsAvantProchainTir = TEMPS_ENTRE_DEUX_TIRS;
				SonManager.getSonManager().jouerTirJoueur();
				Environnement.getEnvironnement().getTirsJoueur().add( new Tir( (int)x, (int)y,0.3f,Tir.TIR_JOUEUR) );
			}

		}
	}
	
	/**
	 * Met a jour les positions
	 */
	public void raffraichir( long temps )
	{
		/*--*/
		// CONTROLE CLAVIER
		/*--*/
		
		// SI LE JOUEUR S'EST FAIT TOUCHE ON LE FREEZE
		if( tempsFreeze > 0 )
		{
			tempsFreeze 	   -= temps;
			vitesse				= 0f;
			premierDeplacement 	= 0;
		}
		// SINON IL PEUT JOUER
		else
		{
		
			//SI LA BALLE EST COLLEE A LA RAQUETTE
			if( Environnement.getEnvironnement().getBalle().getEtat() == Balle.ACCROCHEE_AU_JOUEUR )
			{
				// CHANGEMENT DE L'ANGLE DE LA BALLE SI ATTACHEE AU JOUEUR
				if( ClavierManager.getClavierManager().estHaut() || ClavierManager.getClavierManager().estBas() )
				{
					//SI BALLE PART VERS LA DROITE
					if( Environnement.getEnvironnement().getBalle().getAngle() == -45 )
					{
						Environnement.getEnvironnement().getBalle().setAngle( -135 );
					}
					//SI BALLE PART VERS LA GAUCHE
					else if( Environnement.getEnvironnement().getBalle().getAngle() == -135 )
					{
						Environnement.getEnvironnement().getBalle().setAngle( -45 );
					}
					
					tempsAffichageLanceBalle = 2000;
					
					SonManager.getSonManager().jouerChangerSensLancementBalle();
					
					ClavierManager.getClavierManager().validerHaut();
					ClavierManager.getClavierManager().validerBas();
				}
				
				// SI ON TIR CALCUL DU NOUVEL ANGLE
				if( ClavierManager.getClavierManager().estTir() )
				{
					Environnement.getEnvironnement().getBalle().setAngle( Environnement.getEnvironnement().getBalle().getAngle() + getVitesseX()*20 );
					Environnement.getEnvironnement().getBalle().setEtat( Balle.LIBRE );
					ClavierManager.getClavierManager().validerTir();
					
					tempsAffichageLanceBalle = 0;
				}
			}
		
			
			// SI BOUTON GAUCHE
			if( ClavierManager.getClavierManager().estGauche() )
			{
				// SI LE DERNIER DEPLACEMENT ETAIT DANS L'AUTRE SENS ON REPART A LA VITESSE REDUITE
				if( angle == 0 )
					premierDeplacement = 0;
				
				// SI ON A PAS ENCORE ATTEINT LA VITESSE MAXIMALE
				if( premierDeplacement < 1000 )
				{
					vitesse 			= 0.75f * (premierDeplacement / 1000.0f) + 0.35f;
					premierDeplacement 	+= temps;
				}
				else
					vitesse = 1.1f;
				
				angle = 180;
			}
			// SI BOUTON DROITE
			else if( ClavierManager.getClavierManager().estDroite() ) 
			{
				// SI LE DERNIER DEPLACEMENT ETAIT DANS L'AUTRE SENS ON REPART A LA VITESSE REDUITE
				if( angle == 180 )
					premierDeplacement = 0;
				
				// SI ON A PAS ENCORE ATTEINT LA VITESSE MAXIMALE
				if( premierDeplacement < 1000 )
				{
					vitesse 			= 0.75f * (premierDeplacement / 1000.0f) + 0.35f;
					premierDeplacement 	+= temps;
				}
				else
					vitesse = 1.1f;
				
				angle = 0;
			}
			// SI NI GAUCHE NI DROITE
			else
			{
				vitesse = 0;
				premierDeplacement = 0;
			}
		}
		
		// GESTION DES TEMPS AFFICHAGE DIRECTION
		if( tempsAffichageLanceBalle > 0 )
			tempsAffichageLanceBalle -= temps;
		else if( tempsAffichageLanceBalle < 0 || Environnement.getEnvironnement().getBalle().getEtat() == Balle.LIBRE )
			tempsAffichageLanceBalle = 0;
		
		// MODIFICATION POSITION
		super.raffraichir(temps);
		
		// GESTION DES TIRS
		gestionTirs(temps);
		
		// CONTROLE DE LA POSITION
		if( x < 6 + getWidth()/2 )
			x = 6 + getWidth()/2;
		else if( x > 1024 - 6 - getWidth()/2)
			x = 1024 - 6 - getWidth()/2;
	}
	
	/**
	 * Dessine la raquette
	 */
	public void dessiner( Graphics2D graphics2D )
	{
		// AFFICHAGE DES EFFETS
		if( tempsFreeze > 0 )
			graphics2D.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, .5f) );
		
		// AFFICHAGE DE LA RAQUETTE
        BufferedImage bufferedImage = new BufferedImage( ImageManager.getImagemanager().getImageJoueur(1).getIconWidth() , getHeight(), BufferedImage.TYPE_INT_ARGB );
        Graphics g 					= bufferedImage.createGraphics();
        g.drawImage( ImageManager.getImagemanager().getImageJoueur(1).getImage(), 0, 0, null );
        g.dispose();
        graphics2D.setPaint( new TexturePaint(bufferedImage, new Rectangle( (int)x, (int)y ,ImageManager.getImagemanager().getImageJoueur(1).getIconWidth(),getHeight() ) ) );
        graphics2D.fillRect( (int)x - getWidth()/2 + ImageManager.getImagemanager().getImageJoueur(0).getIconWidth(), (int)y, tailleRaquette, getHeight() );	
        
        // AFFICHAGE DES COTES
		graphics2D.drawImage( ImageManager.getImagemanager().getImageJoueur(0).getImage(), (int)x - getWidth()/2, (int)y, null );
		graphics2D.drawImage( ImageManager.getImagemanager().getImageJoueur(2).getImage(), (int)x - getWidth()/2 + ImageManager.getImagemanager().getImageJoueur(0).getIconWidth() + tailleRaquette, (int)y, null );
		
		// AFFICHAGE DES DIRECTIONS DE LANCE DE BALLE
		if( Environnement.getEnvironnement().getBalle().getAngle() == -45 && tempsAffichageLanceBalle > 0 )
			graphics2D.drawImage( ImageManager.getImagemanager().getImageJoueur(3).getImage(), (int)x, (int)y - 72, null );
		else if( Environnement.getEnvironnement().getBalle().getAngle() == -135 && tempsAffichageLanceBalle > 0  )
			graphics2D.drawImage( ImageManager.getImagemanager().getImageJoueur(4).getImage(), (int)x - 72, (int)y - 72, null );
		
		graphics2D.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, 1f) );
	}
}
