package elements;

import java.awt.Graphics2D;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import javax.swing.ImageIcon;

/**
 * 
 * @author Jerome Baudoux : http://www.jerome-baudoux.com
 *
 */
public class ObjetDessinable implements Raffraichissable, Dessinable
{
	protected float x;
	protected float y;
	
	protected ImageIcon img;
	
	
	/**
	 * Objet pouvant se dessinner
	 * @param valeurX position en x
	 * @param valeurY position en y
	 */
	public ObjetDessinable( int valeurX, int valeurY)
	{
		x = valeurX;
		y = valeurY;
	}
	

	/**
	 * Recupre la position de l'objet dessinable
	 * @return position en x
	 */
	public int getX()
	{
		return (int)x;
	}
	
	
	/**
	 * Recupre la position de l'objet dessinable
	 * @return position en y
	 */
	public int getY()
	{
		return (int)y;
	}
	
	
	/**
	 * Recupre la taille de l'objet dessinable
	 * @return largeur
	 */
	public int getWidth()
	{
		return img.getImage().getWidth(null);
	}
	
	
	/**
	 * Recupre la taille de l'objet dessinable
	 * @return hauteur
	 */
	public int getHeight()
	{
		return img.getImage().getHeight(null);
	}	
	
	
	/**
	 * Recupre l'air de l'objet
	 * @return ait de l'objet par defaut un retangle de la taille de l'image
	 */
	public Area getArea()
	{
		return new Area( new Rectangle2D.Double( x, y, img.getIconWidth(), img.getIconHeight()));
	}
	
	
	/**
	 * Modifie la position de l'objet dessinable
	 * @param valeurX position en x
	 */
	public void setX( int valeurX )
	{
		x = valeurX;
	}	
	
	
	/**
	 * Modifie la position de l'objet dessinable
	 * @param valeurY position en y
	 */
	public void setY( int valeurY )
	{
		y = valeurY;
	}
	
	
	/**
	 * Met la jour les valeurs
	 * @param temps
	 */
	public void raffraichir( long temps ){}	
	
	
	/**
	 * Dessine l'objet
	 * @param graphics2D graphique contenneur
	 */
	public void dessiner(Graphics2D graphics2D) 
	{
		graphics2D.drawImage( img.getImage(), (int)x, (int)y, null  );
	}


}
