package elements;

import controles.Environnement;

public class ObjetMobile extends ObjetDessinable
{
	protected float vitesse;
	protected float angle;
	
	/**
	 * Objet ayant une trajectoire definie par une vitesse et un angle
	 * @param valeurX position en x
	 * @param valeurY position en y
	 * @param valeurVitesse vitesse de dplacement
	 * @param valeurAngle angle de dplacement
	 */
	public ObjetMobile( int valeurX, int valeurY, float valeurVitesse, float valeurAngle )
	{
		super(valeurX,valeurY);
		
		vitesse = valeurVitesse;
		angle 	= valeurAngle;
	}
	
	
	
	/**
	 * Renvoie la vitesse de l'objet
	 * @return vitesse
	 */
	public float getVitesse()
	{
		return vitesse;
	}

	
	
	/**
	 * Renvoie la vitesse de l'objet en x
	 * @return vitesse en X
	 */	
	public float getVitesseX()
	{
		return (float)Math.cos( Math.toRadians( angle ))*vitesse;
	}	

	
	
	/**
	 * Renvoie la vitesse de l'objet en Y
	 * @return vitesse en Y
	 */	
	public float getVitesseY()
	{
		return (float)Math.sin( Math.toRadians( angle ))*vitesse;
	}	
		
	
	
	/**
	 * Renvoie l'angle de l'objet
	 * @return angle
	 */
	public float getAngle()
	{
		return angle;
	}
	
	
	
	/**
	 * Defini la vitesse de l'objet
	 * @param valeurVitesse vitesse de l'objet
	 */
	public void setVitesse( float valeurVitesse )
	{
		vitesse = valeurVitesse;
	}

	
	
	/**
	 * Defini la vitesse de l'objet
	 * @param valeurVitesse vitesse de l'objet
	 */
	public void setVitesseXY( float valeurVitesseX, float valeurVitesseY )
	{
		// MET A JOUR L'ANGLE
		float angleTemp = (float)Math.toDegrees( Math.acos(  Math.abs(valeurVitesseX)/(Math.sqrt( valeurVitesseX*valeurVitesseX + valeurVitesseY*valeurVitesseY)) ) );
		
		if( valeurVitesseX < 0 && valeurVitesseY < 0)
			angle = angleTemp + 180;
		else if( valeurVitesseX < 0 && valeurVitesseY >= 0)
			angle = 180 - angleTemp;
		else if( valeurVitesseX >= 0 && valeurVitesseY < 0)
			angle = -angleTemp;
		else
			angle = angleTemp;
		
		vitesse = (float)(Math.sqrt( valeurVitesseX*valeurVitesseX + valeurVitesseY*valeurVitesseY));
	}	
	
	
	
	/**
	 * Defini l'angle de l'objet
	 * @param valeurAngle angle de l'objet
	 */
	public void setAngle( float valeurAngle )
	{
		angle = valeurAngle;
	}
	
	
	
	/**
	 * Met la jour les valeurs de position
	 * @param temps
	 */
	public void raffraichir( long temps,float alea )
	{
		x += (float) (temps * Math.cos( Math.toRadians( angle ) ) * vitesse * Environnement.getEnvironnement().getVitesseDuJeu())+(temps*alea/1.0);
		y += (float) (temps * Math.sin( Math.toRadians( angle ) ) * vitesse * Environnement.getEnvironnement().getVitesseDuJeu());
	}
	
	/**
	 * Met la jour les valeurs de position
	 * @param temps
	 */
	public void raffraichir( long temps)
	{
		x += (float) (temps * Math.cos( Math.toRadians( angle ) ) * vitesse * Environnement.getEnvironnement().getVitesseDuJeu());
		y += (float) (temps * Math.sin( Math.toRadians( angle ) ) * vitesse * Environnement.getEnvironnement().getVitesseDuJeu());
	}
}
