package elements.ennemis;

import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.Area;

import controles.Environnement;
import controles.ImageManager;
import controles.SonManager;
import elements.Balle;
import elements.Bonus;
import elements.Destructible;
import elements.ObjetDessinable;
import elements.Tir;

public class Ennemi extends ObjetDessinable implements Destructible
{
	private static int TEMPS_ENTRE_DEUX_TIRS = 3000;
	
	/**
	 * energie de l'ennemi
	 */
	int energie;
	
	
	/**
	 * energie de l'ennemi
	 */
	int energieMax;
	
	/**
	 * niveau de l'ennemi
	 */
	int level;
	
	/**
	 * Points donns par l'ennemi
	 */
	int points;
	

	/**
	 * Temps avant que l'ennemi puisse tirer
	 */
	int tempsAvantProchainTir;
	
	
	/**
	 * Bonus de l'ennemi
	 */
	int bonus;
	
	
	/**
	 * Ennemi
	 * @param valeurX valeur de X
	 * @param valeurY valeur de Y
	 * @param lvl niveau de l'ennemi
	 * @param b bonus
	 */
	public Ennemi(int valeurX, int valeurY, int lvl, int en, int b) 
	{
		super(valeurX, valeurY);
		img = ImageManager.getImagemanager().getImageEnnemi( lvl - 1 );
		
		energie 	= en;
		energieMax 	= en;
		level		= lvl;
		points		= lvl * 5 + 5;
		bonus		= b;
		
		tempsAvantProchainTir = TEMPS_ENTRE_DEUX_TIRS;
	}
	
	
	// GETTERS
	
	/**
	 * recupre l'ennergie de l'objet
	 * @return
	 */
	public int getEnergie() 
	{
		return energie;
	}
	
	
	public int getPoints()
	{
		return points;
	}
	
	
	// FONCTIONS
	
	/**
	 * Enlve de l'ennergie  l'objet
	 */
	public void enleverEnergie() 
	{
		enleverEnergie(1);
	}
	
	/**
	 * Enlve de l'ennergie  l'objet
	 */
	public void enleverEnergie( int e ) 
	{
		energie -= e;
	}	

	
	/**
	 * Savoir si l'objet est detruit
	 * @return vrai si l'objet est mort
	 */
	public boolean estMort() 
	{
		return energie <= 0;
	}
	
	
	/**
	 * Detruis l'objet
	 */
	public void tuer()
	{
		// LACHE LE BONUS
		if( bonus != 0)
			Environnement.getEnvironnement().getBonus().add( new Bonus( (int)(x + getWidth()/2) , (int)(y + getHeight()/2), 0.15f, bonus ));
		
		// MAJ DES POINTS
		Environnement.getEnvironnement().ajouerPoints( getPoints() );
		
		// JOUE LE SON
		if( energie <= 0 )
			SonManager.getSonManager().jouerMortEnnemi();	
	}
	
	/**
	 * Savoir si l'objet est pret a tirer
	 * @return vrai si l'objet est pret a tirer
	 */
	public boolean pretaTirer() 
	{
		return tempsAvantProchainTir <= 0;
	}
	
	
	
	/**
	 * Ordonne a l'ennemi de tirer
	 */
	public void tirer()
	{
		tempsAvantProchainTir = TEMPS_ENTRE_DEUX_TIRS;
		Environnement.getEnvironnement().getTirsEnnemis().add( new Tir( (int)(x + getWidth()/2), (int)(y + getHeight()/2), 0.5f, Tir.TIR_ENNEMI) );
	}
	
	
	/**
	 * Traite la collision entre l'ennemi et la balle
	 * @param b balle
	 */
	public boolean collision( Balle b )
	{
		return collision(b,true);
	}
	
	/**
	 * Traite la collision entre l'ennemi et la balle
	 * @param b balle
	 * @param degats l'ennemi subira il des degars
	 */
	public boolean collision( Balle b , boolean degats)
	{
		int[] posX;
		int[] posY;
		
		// RECUPERATION DES AIRES
		Area airBalle	= b.getArea();

		posX 			= new int[] {(int)x,(int)(x+getWidth()/2),(int)x + getWidth()};
		posY 			= new int[] {(int)y,(int)(y+getHeight()/2),(int)(y)};
		Area airHaut 	= new Area	( new Polygon(posX, posY, 3));

		posX 			= new int[] {(int)x,(int)(x+getWidth()/2),(int)x + getWidth()};
		posY 			= new int[] {(int)y + getHeight(),(int)(y+getHeight()/2),(int)(y+getHeight())};
		Area airBas 	= new Area	( new Polygon(posX, posY, 3));		
		
		posX 			= new int[] {(int)x,(int)(x+getWidth()/2),(int)x};
		posY 			= new int[] {(int)y,(int)(y+getHeight()/2),(int)(y+getHeight())};
		Area airGauche 	= new Area	( new Polygon(posX, posY, 3));

		posX 			= new int[] {(int)x + getWidth() ,(int)(x+getWidth()/2),(int)x + getWidth()};
		posY 			= new int[] {(int)y,(int)(y+getHeight()/2),(int)(y+getHeight())};
		Area airDroite 	= new Area	( new Polygon(posX, posY, 3));

		// INTERSECTION DES AIRES
		airGauche.intersect( airBalle );
		airDroite.intersect( airBalle );
		airHaut.intersect  ( airBalle );
		airBas.intersect   ( airBalle );
		
		// AIRES ( Largeur * hauteur ) DES AREA
		double airGaucheVal = airGauche.getBounds2D().getWidth() 	* airGauche.getBounds2D().getHeight();
		double airDroiteVal = airDroite.getBounds2D().getWidth() 	* airDroite.getBounds2D().getHeight();
		double airHautVal 	= airHaut.getBounds2D().getWidth() 		* airHaut.getBounds2D().getHeight();
		double airBasVal 	= airBas.getBounds2D().getWidth() 		* airBas.getBounds2D().getHeight();
		
		// BOOLEEN DEFINISSANT LE REBOND
		boolean collisionGauche = false;
		boolean collisionDroite = false;
		boolean collisionHaut 	= false;
		boolean collisionBas 	= false;
		
		// HAUT GAUCHE -> BAS DROITE
		if( b.getVitesseY() > 0 &&  b.getVitesseX() > 0)
		{
			// COLLISION HAUT GAUCHE
			if( ! airHaut.isEmpty() && ! airGauche.isEmpty() )
			{
				if( airGaucheVal > airHautVal )
					collisionGauche = true;
				else
					collisionHaut 	= true;
			}
			// COLLISION HAUT DROITE
			else if( ! airHaut.isEmpty() && ! airDroite.isEmpty())
			{
					collisionHaut 	= true;	
			}
			// COLLISION Bas GAUCHE
			if( ! airBas.isEmpty() && ! airGauche.isEmpty())
			{
					collisionGauche = true;
			}
			// COLLISION BAS DROITE
			else if( ! airBas.isEmpty()  && ! airDroite.isEmpty())
			{
				if( airDroiteVal > airBasVal )
					collisionHaut = true;
				else
					collisionGauche = true;	
			}		
			// SI COLLISION EN HAUT
			else if( ! airHaut.isEmpty() )
			{
				collisionHaut = true;
			}
			// SI COLLISION EN BAS
			else if( ! airBas.isEmpty() )
			{
				collisionGauche = true;
			}
			// SI COLLISION A GAUCHE
			else if( ! airGauche.isEmpty() )
			{
				collisionGauche = true;
			}
			// SI COLLISION A DROITE
			else if( ! airDroite.isEmpty() )
			{
				collisionHaut = true;
			}
			
			
			
		}
		// BAS GAUCHE -> HAUT DROITE
		else if( b.getVitesseY() <= 0 &&  b.getVitesseX() > 0 )
		{
			
			
			// COLLISION HAUT GAUCHE
			if( ! airHaut.isEmpty() && ! airGauche.isEmpty() )
			{
					collisionGauche = true;
			}
			// COLLISION HAUT DROITE
			else if( ! airHaut.isEmpty() && ! airDroite.isEmpty())
			{
				if( airHautVal > airDroiteVal )
					collisionGauche = true;
				else
					collisionBas = true;	
			}
			// COLLISION Bas GAUCHE
			if( ! airBas.isEmpty() && ! airGauche.isEmpty())
			{
				if( airGaucheVal > airBasVal )
					collisionGauche = true;
				else
					collisionBas = true;	
			}
			// COLLISION BAS DROITE
			else if( ! airBas.isEmpty()  && ! airDroite.isEmpty())
			{
				collisionBas = true;	
			}		
			// SI COLLISION EN HAUT
			else if( ! airHaut.isEmpty() )
			{
				collisionGauche = true;
			}
			// SI COLLISION EN BAS
			else if( ! airBas.isEmpty() )
			{
				collisionBas = true;
			}
			// SI COLLISION A GAUCHE
			else if( ! airGauche.isEmpty() )
			{
				collisionGauche = true;
			}
			// SI COLLISION A DROITE
			else if( ! airDroite.isEmpty() )
			{
				collisionBas = true;
			}
			
			
			
		}
		// HAUT DROITE -> BAS GAUCHE
		else if( b.getVitesseY() > 0 &&  b.getVitesseX() <= 0 )
		{
			
			
			
			// COLLISION HAUT GAUCHE
			if( ! airHaut.isEmpty() && ! airGauche.isEmpty() )
			{
					collisionHaut = true;
			}
			// COLLISION HAUT DROITE
			else if( ! airHaut.isEmpty() && ! airDroite.isEmpty())
			{
				if( airHautVal > airDroiteVal )
					collisionHaut = true;
				else
					collisionDroite = true;	
			}
			// COLLISION Bas GAUCHE
			if( ! airBas.isEmpty() && ! airGauche.isEmpty())
			{
				if( airGaucheVal > airBasVal )
					collisionHaut = true;
				else
					collisionDroite = true;	
			}
			// COLLISION BAS DROITE
			else if( ! airBas.isEmpty()  && ! airDroite.isEmpty())
			{
				collisionDroite = true;	
			}		
			// SI COLLISION EN HAUT
			else if( ! airHaut.isEmpty() )
			{
				collisionHaut = true;
			}
			// SI COLLISION EN BAS
			else if( ! airBas.isEmpty() )
			{
				collisionDroite = true;
			}
			// SI COLLISION A GAUCHE
			else if( ! airGauche.isEmpty() )
			{
				collisionHaut = true;
			}
			// SI COLLISION A DROITE
			else if( ! airDroite.isEmpty() )
			{
				collisionDroite = true;
			}
			
			
			
			
			
		}
		// BAS DOITE - > HAUT GAUCHE
		else if( b.getVitesseY() <= 0 &&  b.getVitesseX() <= 0 )
		{
			
			
			
			
			// COLLISION HAUT GAUCHE
			if( ! airHaut.isEmpty() && ! airGauche.isEmpty() )
			{
				if( airGaucheVal > airHautVal )
					collisionBas = true;
				else
					collisionDroite 	= true;
			}
			// COLLISION HAUT DROITE
			else if( ! airHaut.isEmpty() && ! airDroite.isEmpty())
			{
					collisionDroite 	= true;	
			}
			// COLLISION Bas GAUCHE
			if( ! airBas.isEmpty() && ! airGauche.isEmpty())
			{
					collisionBas = true;
			}
			// COLLISION BAS DROITE
			else if( ! airBas.isEmpty()  && ! airDroite.isEmpty())
			{
				if( airDroiteVal > airBasVal )
					collisionDroite = true;
				else
					collisionBas = true;	
			}		
			// SI COLLISION EN HAUT
			else if( ! airHaut.isEmpty() )
			{
				collisionDroite = true;
			}
			// SI COLLISION EN BAS
			else if( ! airBas.isEmpty() )
			{
				collisionBas = true;
			}
			// SI COLLISION A GAUCHE
			else if( ! airGauche.isEmpty() )
			{
				collisionBas = true;
			}
			// SI COLLISION A DROITE
			else if( ! airDroite.isEmpty() )
			{
				collisionDroite = true;
			}
			
			
			
			
		}
		
		// SI IL Y A EU COLLISION
		if( collisionHaut || collisionBas || collisionGauche || collisionDroite)
		{
			if( collisionHaut )
			{
				b.setY( (int)y - b.getHeight() );
				b.setVitesseXY( b.getVitesseX(), - b.getVitesseY());
			}
			else if( collisionBas )
			{
				b.setY( (int)y + getHeight() );
				b.setVitesseXY( b.getVitesseX(), - b.getVitesseY());
			}
			else if( collisionGauche )
			{
				b.setX( (int)x - b.getWidth() );		
				b.setVitesseXY( - b.getVitesseX(), b.getVitesseY());
			}
			else if( collisionDroite )
			{
				b.setX( (int)x + getWidth() );
				b.setVitesseXY( - b.getVitesseX(), b.getVitesseY());
			}
			
			//EFFECTUE LES OPERATIONS
			if( degats )
				enleverEnergie();	
			
			// JOUE LES SONS NECESSAIRES
			SonManager.getSonManager().jouerToucheEnnemi();
			
			if( energie <= 0 )
				tuer();
			
			return true;
		}
		return false;
	}
	
	/**
	 * Collision entre un ennemi et un tir
	 * @param t tir
	 */
	public void collision( Tir t )
	{
		collision( t, true );
	}
	
	/**
	 * Collision entre un ennemi et un tir
	 * @param t tir
	 * @param degats subira il des degats
	 */
	public boolean collision( Tir t, boolean degats)
	{	
		Area aire = getArea();
		aire.intersect( t.getArea() );
		
		// SI il y a collision
		if( ! aire.isEmpty() )
		{
			t.tuer();
			
			//EFFECTUE LES OPERATIONS
			if( degats )
				enleverEnergie();	
			
			// JOUE LES SONS NECESSAIRES
			SonManager.getSonManager().jouerToucheEnnemi();
			
			if( energie <= 0 )
				tuer();
			
			return true;
		}
		return false;
	}
	
	
	public void raffraichir( long temps )
	{
		super.raffraichir(temps);
		
		// GESTION DES TIRS
		if( tempsAvantProchainTir > 0 )
			tempsAvantProchainTir -= temps;
	}
	
	
	
	/**
	 * Dessine l'objet
	 * @param graphics2D graphique contenneur
	 */
	public void dessiner(Graphics2D graphics2D) 
	{
		// DESSINE L'ENNEMI EN % D'ENERGIE RESTANTE
		graphics2D.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, energie/((float)energieMax) ) );
		graphics2D.drawImage   ( img.getImage(), (int)x, (int)y, null  );
		graphics2D.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, 1f ) );
	}	

}
