package jeu;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferStrategy;
import controles.*;
import elements.Balle;

/**
 * 
 * @author Simon Mahieux et Jerome Baudoux
 *
 */
public class Jeu 
{
	private OptionsManager om;

	/**
	 * JEU
	 */
	public Jeu()
	{
		Environnement.reset();
		NiveauManager.init();
	}
	
	/**
	 * commencer le jeu au niuveauime niveau
	 * @param niveau numero du niveau
	 */
	public Jeu( int niveau )
	{
		Environnement.reset();
		NiveauManager.init(niveau-1);
	}
	
	
	
	/**
	 * BOUCLE DE JEU
	 * @param strategy buffer d'affichage
	 */
	public void run(BufferStrategy strategy)
	{
		long tempsDepuisLaDerniereFrame;
		long tempsDerniereFrame = System.currentTimeMillis();
		
		om = OptionsManager.getInstance();

		/*  BOUCLE DE JEU  */
		while ( ! Environnement.getEnvironnement().finJeu() ) 
		{
			// ------------------------
			// GESTION DU TEMPS
			// ------------------------
			tempsDepuisLaDerniereFrame 	= System.currentTimeMillis() - tempsDerniereFrame;
			tempsDerniereFrame  		= System.currentTimeMillis();
			
			// BLOQUE LE TEMPS ENTRE DEUX FRAMES A 100 SI SUPERIEUR LE JEU RAMMERA
			if( tempsDepuisLaDerniereFrame > 50 )
				tempsDepuisLaDerniereFrame = 50;
			
			// CONTEXTE GRAPHIQUE
			Graphics2D graphics2D = (Graphics2D) strategy.getDrawGraphics();	
			
			// CHANGEMENT DE RESOLUTION
			if( om.get("width") != 1024 && om.get("height") != 768)
			{
				graphics2D.scale(om.get("width")/1024.0, om.get("height")/768.0);
				
				graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING		, RenderingHints.VALUE_RENDER_QUALITY			);
				graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING	, RenderingHints.VALUE_COLOR_RENDER_QUALITY	);	
				graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING		, RenderingHints.VALUE_ANTIALIAS_ON		);
				graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON	);
			}
			else
			{
				graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING		, RenderingHints.VALUE_RENDER_SPEED			);
				graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING	, RenderingHints.VALUE_COLOR_RENDER_SPEED);	
				graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING		, RenderingHints.VALUE_ANTIALIAS_OFF		);
				graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF	);	
			}
			
			try
			{
				graphics2D.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_OFF);
				graphics2D.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_DISABLE);
			}
			catch( Exception e ){}
			
			// CALCULS DE COLLISIONS DE PLACEMENT ET AFFICHAGE				
			raffraichir( tempsDepuisLaDerniereFrame );
			collisions ();
			destruction();
			dessinner  ( graphics2D );

			// AFFICHAGE DU GRAPHIQUE
			graphics2D.dispose();
			strategy.show();
			
			// PERMET DE REPOSER LE PORSS
			try 
			{
				Thread.sleep(1);
			} 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}	
		}	
	}
	
	

	/**
	 * Gere les Destructions
	 */
	private void destruction()
	{
		// SI LE JEU N'EST PAS EN PAUSE
		if( Environnement.getEnvironnement().estJeu())
		{
			
			// DETRUIT LES ENNEMIS
			for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size(); i++)
				if( NiveauManager.getNiveauManager().getEnnemis().get(i).estMort())
				{
					NiveauManager.getNiveauManager().getEnnemis().remove(i);
					i--;
				}
			
			
			// DETRUIT LES TIRS ENNEMIS
			for(int i=0; i<Environnement.getEnvironnement().getTirsEnnemis().size(); i++)
				if( Environnement.getEnvironnement().getTirsEnnemis().get(i).estMort())
				{
					Environnement.getEnvironnement().getTirsEnnemis().remove(i);
					i--;
				}
			
			
			// DETRUIT LES TIRS JOUEUR
			for(int i=0; i<Environnement.getEnvironnement().getTirsJoueur().size(); i++)
				if( Environnement.getEnvironnement().getTirsJoueur().get(i).estMort())
				{
					Environnement.getEnvironnement().getTirsJoueur().remove(i);
					i--;
				}
			
			
			// DETRUIT LES BONUS
			for(int i=0; i<Environnement.getEnvironnement().getBonus().size(); i++)
				if( Environnement.getEnvironnement().getBonus().get(i).estMort())
				{
					Environnement.getEnvironnement().getBonus().remove(i);
					i--;
				}
			
		}
	}	
	
	
	/**
	 * Gere les Collsions
	 */
	private void collisions()
	{
		// SI LE JEU N'EST PAS EN PAUSE
		if( Environnement.getEnvironnement().estJeu())
		{
			// COLLISION JOUEUR / BALLE
			Environnement.getEnvironnement().getJoueur().collision(Environnement.getEnvironnement().getBalle());
			
			// COLLISION ENNEMIS / BALLE
			//for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size() && (NiveauManager.getNiveauManager().getEnnemis().get(i).collision(Environnement.getEnvironnement().getBalle()) == false); i++){}

			boolean collisionBalleEnnemi = false;
			for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size(); i++)
				if( NiveauManager.getNiveauManager().getEnnemis().get(i).collision(Environnement.getEnvironnement().getBalle()) )
					collisionBalleEnnemi = true;
			
			if( ! collisionBalleEnnemi )
				Environnement.getEnvironnement().getBalle().setLast();
			
			// COLLISION ENNEMIS / TIRS JOUEUR
			for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size(); i++)
				for(int j=0; j<Environnement.getEnvironnement().getTirsJoueur().size(); j++)
					NiveauManager.getNiveauManager().getEnnemis().get(i).collision(Environnement.getEnvironnement().getTirsJoueur().get(j));
			
			// COLLISION JOUEUR / TIRS ENNEMIS
			for(int j=0; j<Environnement.getEnvironnement().getTirsEnnemis().size(); j++)
				Environnement.getEnvironnement().getJoueur().collision(Environnement.getEnvironnement().getTirsEnnemis().get(j));
			
			// COLLISION JOUEUR / BONUS
			for(int j=0; j<Environnement.getEnvironnement().getBonus().size(); j++)
				Environnement.getEnvironnement().getJoueur().collision(Environnement.getEnvironnement().getBonus().get(j));
			
			// COLLISION BALLE / TIRS
			if( Environnement.getEnvironnement().getBalle().getEtat() == Balle.LIBRE )
			{
				for(int i=0; i<Environnement.getEnvironnement().getTirsEnnemis().size(); i++)
					Environnement.getEnvironnement().getBalle().collision(Environnement.getEnvironnement().getTirsEnnemis().get(i));
				for(int i=0; i<Environnement.getEnvironnement().getTirsJoueur().size(); i++)
					Environnement.getEnvironnement().getBalle().collision(Environnement.getEnvironnement().getTirsJoueur().get(i));
			}

		}
	}
	
	
	/**
	 * Rafraichi les positions
	 * 
	 * @param temps
	 */
	private void raffraichir(long temps)
	{
		Environnement.getEnvironnement().raffraichir(temps);
		
		// SI LE JEU N'EST PAS EN PAUSE
		if( Environnement.getEnvironnement().estJeu() )
		{
			Environnement.getEnvironnement().getJoueur().raffraichir(temps);
			
			for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size(); i++)
				NiveauManager.getNiveauManager().getEnnemis().get(i).raffraichir(temps);
			
			for(int i=0; i<Environnement.getEnvironnement().getTirsEnnemis().size(); i++)
				Environnement.getEnvironnement().getTirsEnnemis().get(i).raffraichir(temps);

			for(int i=0; i<Environnement.getEnvironnement().getTirsJoueur().size(); i++)
				Environnement.getEnvironnement().getTirsJoueur().get(i).raffraichir(temps);
	
			for(int i=0; i<Environnement.getEnvironnement().getBonus().size(); i++)
				Environnement.getEnvironnement().getBonus().get(i).raffraichir(temps);
			
			Environnement.getEnvironnement().getBalle().raffraichir(temps);
		}
	}
	
	
	/**
	 * Redessine tout
	 * 
	 * @param g
	 */
	private void dessinner(Graphics2D graphics2D)
	{
		// DESSINE L'IMAGE DE FOND
		if( ImageManager.getImagemanager().getImageNiveau() != null )
			graphics2D.drawImage( ImageManager.getImagemanager().getImageNiveau().getImage(), 0, 0, null);
		else
			graphics2D.fillRect(0, 0, 1024, 768);
		
		/* DESSINNE LES ELEMENTS */
		Environnement.getEnvironnement().getJoueur().dessiner(graphics2D);
		
		for(int i=0; i<NiveauManager.getNiveauManager().getEnnemis().size(); i++)
			NiveauManager.getNiveauManager().getEnnemis().get(i).dessiner(graphics2D);
		
		for(int i=0; i<Environnement.getEnvironnement().getTirsEnnemis().size(); i++)
			Environnement.getEnvironnement().getTirsEnnemis().get(i).dessiner(graphics2D);

		for(int i=0; i<Environnement.getEnvironnement().getTirsJoueur().size(); i++)
			Environnement.getEnvironnement().getTirsJoueur().get(i).dessiner(graphics2D);
		
		for(int i=0; i<Environnement.getEnvironnement().getBonus().size(); i++)
			Environnement.getEnvironnement().getBonus().get(i).dessiner(graphics2D);
		
		Environnement.getEnvironnement().getBalle().dessiner(graphics2D);
		
		Environnement.getEnvironnement().dessiner(graphics2D);
	}
}









