package jeu;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.image.BufferStrategy;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import controles.ClavierManager;
import controles.ImageManager;
import controles.OptionsManager;
import controles.ScreenManager;
import controles.SonManager;
import menus.*;


/**
 * 
 * @author Simon Mahieux et Jerome Baudoux
 *
 */
public class Projet extends Canvas
{
	private static final long serialVersionUID = 1L;
	
	private BufferStrategy 	strategy;
	private JFrame 			frame;
	private ImageIcon 		icon;
	
	private Menu 			menuCourrant;
	
	private ClavierManager 	clavier;
    private OptionsManager	om;	
	private ScreenManager 	sm;
    
	private Jeu 			partie;
	
	private int				niveauDepart;
	

	
    /**
     * Constructeur
     */
	public Projet( String [] args ) 
	{
		
		// MENUS ET CONTROLES
		clavier 		= ClavierManager.getClavierManager();
		sm 				= new ScreenManager();
		menuCourrant 	= new MenuPrincipal();	
		
		// INITIALISER LA FENETRE
		initialiserFenetre();
	    
		// GESTION DE L'ECOUTEUR CLAVIER
		addKeyListener( clavier );
		requestFocus();	
		
		// CHOIX DU NIVEAU
		if( args.length > 0 )
			niveauDepart = Integer.parseInt( args[0] );
		else
			niveauDepart = -1;

		// Lancer Menu
		run();
	}   
	
	
	/**
	 * Initialise la fenetre
	 */
	private void initialiserFenetre()
	{
		// GESTION DE LA FENETRE
		frame = new JFrame("Shoot Breaker");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.setUndecorated(false);
		
    	// Options
    	om = OptionsManager.getInstance();
    	
		// IMAGES
		icon = new ImageIcon( Toolkit.getDefaultToolkit().getImage("Images/Logo/16.png") );
		frame.setIconImage( icon.getImage() );
		
		// CURSEUR
		setCursor(Toolkit.getDefaultToolkit().createCustomCursor( Toolkit.getDefaultToolkit().getImage("Images/Divers/mouse.png") , new Point(16, 16), "mouse"));
		

		//CONTENU GRAPHIQUE DE LA FENETRE
		JPanel panel = (JPanel) frame.getContentPane();
		panel.setPreferredSize(new Dimension(om.get("width"), om.get("height")));
		panel.setLayout(null);
		panel.add(this);
		setBounds(0, 0, om.get("width"), om.get("height"));		
		setIgnoreRepaint(true);	
		

		// GESTION DE LA FENETRE
		frame.pack(); 
		frame.setVisible(true);
		
		frame.setLocationRelativeTo( frame.getParent() );
	
		// GESTION DE L'AFFICHAGE
		setIgnoreRepaint( true );  
		createBufferStrategy( 2 ); 
		strategy = getBufferStrategy(); 
		
    	// FullScreen
    	if( om.get("fullScreen") == 1 )
    	{
    		sm.setFullScreen(frame, new DisplayMode( om.get("width"), om.get("height"), 32,  DisplayMode.REFRESH_RATE_UNKNOWN)); 
    		//SI LINUX ON DOIT RECREER LE BUFFER
    		//if ( System.getProperty("file.separator").equals( "\\" ) ) 
    		//{
    			createBufferStrategy( 2 ); 	
    		//}
    	}	
		
    	requestFocus();
	}
	
	/**
	 * Gere les successions de menus
	 */
	private void gererMenus()
	{	
		// SI LE MENU EST DE TYPE MENUPRINCIPAL
		if( menuCourrant.getTypeDeMenu().equals("MenuPrincipal") )
		{
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			if( ((MenuPrincipal) menuCourrant).lancerJeu() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				SonManager.getSonManager().stopMusiqueMenu();
				// LANCE LE JEU TOTALEMENT
				partie = new Jeu();
				partie.run(strategy);
				SonManager.getSonManager().jouerMusiqueMenu();
			}
			// SI L'ORDRE EST DE VOIR LES SCORE
			else if( ((MenuPrincipal) menuCourrant).menuScore() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuScore();
				ClavierManager.getClavierManager().validerTout();
			}
			else if( ((MenuPrincipal) menuCourrant).menuAide() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuAide();
				ClavierManager.getClavierManager().validerTout();
			}			
			// SI L'ORDRE EST DE VOIR LES OPTIONS
			else if( ((MenuPrincipal) menuCourrant).menuOptions() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuOptions();
				ClavierManager.getClavierManager().validerTout();
			}
			// SI L'ORDRE EST LANCE ON QUITTE LE JEU
			else if( ((MenuPrincipal) menuCourrant).quitterJeu() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				sm.restoreFullScreen(frame);
				System.exit(0);
			}			
		}	
		else if( menuCourrant.getTypeDeMenu().equals("MenuScore") )
		{
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			if( ((MenuScore) menuCourrant).menuPrincipal() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuPrincipal();
				ClavierManager.getClavierManager().validerTout();
			}
		}
		else if( menuCourrant.getTypeDeMenu().equals("MenuOptions") )
		{
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			if( ((MenuOptions) menuCourrant).menuPrincipal() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuPrincipal();
				ClavierManager.getClavierManager().validerTout();
			}
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			else if( ((MenuOptions) menuCourrant).menuClavier() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuClavier();
				ClavierManager.getClavierManager().validerTout();
			}
		}
		else if( menuCourrant.getTypeDeMenu().equals("MenuClavier") )
		{
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			if( ((MenuClavier) menuCourrant).menuOptions() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuOptions();
				ClavierManager.getClavierManager().validerTout();
			}
		}
		else if( menuCourrant.getTypeDeMenu().equals("MenuAide") )
		{
			// SI L'ORDRE EST LANCE ON LANCE LE JEU
			if( ((MenuAide) menuCourrant).menuPrincipal() )
			{
				SonManager.getSonManager().jouerBeepMenuValider();
				menuCourrant = new MenuPrincipal();
				ClavierManager.getClavierManager().validerTout();
			}
		}			
	}
	
	
	/**
	 * Lancer le jeu
	 */
	private void run( ) 
	{
		//PRECHARGE LES MANAGERS
		SonManager.getSonManager();
		ImageManager.getImagemanager();
		
		//GESTION DU TEMPS
		long tempsDepuisLaDerniereFrame;
		long tempsDerniereFrame    		= System.currentTimeMillis();
		
		// LANCEMENT DE LA MUSIQUE
		SonManager.getSonManager().jouerMusiqueMenu();
		
		/* 
		 * BOUCLE DE JEU
		 */
		while ( true ) 
		{
			// GESTION DU TEMPS
			tempsDepuisLaDerniereFrame	= System.currentTimeMillis() - tempsDerniereFrame;
			tempsDerniereFrame			= System.currentTimeMillis();
			
			// RECUPERE LA ZONE A DESSINER
			Graphics2D graphics2D 		= (Graphics2D) strategy.getDrawGraphics();
			
			// ICI !!!
			graphics2D.scale(om.get("width")/1024.0, om.get("height")/768.0);
			
			graphics2D.setRenderingHint(RenderingHints.KEY_RENDERING		, RenderingHints.VALUE_RENDER_QUALITY			);
			graphics2D.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING	, RenderingHints.VALUE_COLOR_RENDER_QUALITY	);	
			graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING		, RenderingHints.VALUE_ANTIALIAS_ON		);
			graphics2D.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON	);
			
			// ON DESSINE LE MENU
			menuCourrant.raffraichir( tempsDepuisLaDerniereFrame );
			menuCourrant.dessiner( graphics2D );
			
			
			// SI ON A CHOISI DELIBEREMENT UN NIVEAU
			if( niveauDepart != -1 )
			{
				SonManager.getSonManager().stopMusiqueMenu();
				// LANCE LE JEU EN CHOISISSANT LE NIVEAU
				partie = new Jeu( niveauDepart );
				partie.run(strategy);
				sm.restoreFullScreen(frame);
				System.exit(0);
			}
			
			//Gere les menus
			gererMenus();
			
			// DESSINE L'ECRAN
			graphics2D.dispose();
			strategy.show();	

			// PAUSE POUR LE PROSS
			try {Thread.sleep(1);} catch (InterruptedException e) {e.printStackTrace();}			
		}
	}
	

	/**
	 * MAIN
	 * @param args
	 */
	public static void main(String[] args) 
	{
		new Projet( args );
	}

}
