package son;

import java.io.File;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;

/**
 * Represente un son.
 * 
 * Cette classe n'a pas totalement ete ecrite par Simon Mahieux et Jerome Baudoux, 
 * elle a ete trouvee sur Java.fr.
 * Elle a cependant ete tres (tres) fortement modifiee pour 
 * les besoins du projet.
 * -------------------------------- <br>
 */
public class Son implements Runnable
{

	private AudioInputStream 	lecteurAudio;
	private AudioFormat 		format;
	private Clip 				clip;
	private Thread 				thread;
	private int 				tour;
	private boolean 			pause;
	private boolean				fermerALaFin;
	public  boolean 			ok = true;

   

   /**
    * Construit un son a partir d'un fichier
    * @param fichier Fichier contenant le son
    * @throws SonException Si il y a un probleme de construction du son
    */
   public Son(File fichier)throws Exception
   {
		this.lecteurAudio = AudioSystem.getAudioInputStream(fichier);
		this.format = lecteurAudio.getFormat();
     
		// convertir en PCM
		if((this.format.getEncoding() == AudioFormat.Encoding.ULAW) || (this.format.getEncoding() == AudioFormat.Encoding.ALAW))
		{
			AudioFormat tmp 	= new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, this.format.getSampleRate(), this.format.getSampleSizeInBits() * 2, this.format.getChannels(),this.format.getFrameSize() * 2, this.format.getFrameRate(),true);
			this.lecteurAudio 	= AudioSystem.getAudioInputStream(tmp,this.lecteurAudio);
			this.format 		= tmp;
		}
		         
		DataLine.Info info 	= new DataLine.Info(Clip.class, this.lecteurAudio.getFormat(), ((int)this.lecteurAudio.getFrameLength() * this.format.getFrameSize()));
		this.clip 			= (Clip)AudioSystem.getLine(info);
		         
		//On ouvre le son
	    this.clip.open(this.lecteurAudio);
   }
   

   
   /**
    * Joue le son une fois
    */
   public void jouer()
   {
	   boucle(1);
   }
   
   
   /**
    * Joue le son un tres grand nombre de fois
    */
   public void boucle()
   {
      this.boucle(Integer.MAX_VALUE);
   }
   
   /**
    * Joue le son nbFois fois
    * @param nbFois Nombre de fois que le son est joue
    */
   public void boucle(int nbFois)
   {
      //Si le son n'est pas initialiser, on l'initialise
      if(this.thread == null)
      {
         this.thread = new Thread(this);
         this.thread.start();
      }
      //On va le jouer nbFois fois
      this.tour = nbFois;
   }
   
   

   /**
    * Met le son en pause
    */
   public void pause()
   {
      if(!this.pause)
      {
         this.clip.stop();
         this.pause = true;
      }
   }
   
   
   /**
    * Reprend le son ou il etait rendu (enleve la pause)
    */
   public void reprise()
   {
      if(this.pause)
      {
         pause 	= false;
         this.clip.start();
      }
   }
   
   
   /**
    * Arrete de jouer le son et retour du son au debut
    */
   public void stop()
   {
      this.clip.stop();
      this.clip.setMicrosecondPosition(0);
      this.pause 	= false;
      this.tour 	= 0;
      this.thread 	= null;
   }
   
   
   /**
    * Detruit proprement le son
    */
   public void fermer()
   {
      this.stop();
      this.clip.close();
      this.clip 	= null;
      this.format 	= null;
   }
   
   
   
   /**
    * Indique si le son est en pause
    * @return <b>true </b> si le son est en pause
    */
   public boolean estEnPause()
   {
      return this.pause;
   }
   
   
   /**
    * Indique si le son est entrain d'etre jouer
    * @return <b>true </b> si le son est entrain d'etre joue
    */
   public boolean estEntrainDeJouer()
   {
      return !this.pause && (this.tour > 0);
   }

  
   
   /**
    * Ne pas appeler cette methode
    */
   public void run()
   {
      //Tant que le son est vivant
      while(this.thread != null)
      {
         //Pause de 0.123 seconde
         try
         {
            Thread.sleep(99);
         }
         catch(Exception e){}
         
         //Si on doit jouer le son au moins une fois
         if(this.tour > 0)
         {
            //On lance le son
            this.clip.start();
            
            //pause de 0.099 seconde (le son est jouer pendant ce temps)
            try
            {
               Thread.sleep(99);
            }
            catch(Exception e){}
            
            //Tant que le son n'est pas terminer ou que l'on soit en pause et est vivant
            while((this.thread != null) && (this.clip.isActive() || this.pause))
            {
               //Pause de 0.099 seconde
               try
               {
                  Thread.sleep(99);
               }
               catch(Exception e){break;}
            }
            
            if( this.thread != null )
            {

	            this.clip.stop();
	            this.clip.setMicrosecondPosition(0);
	            this.tour--;
	            
	            if(this.tour < 1)
	            {
	               if(this.fermerALaFin)
	               {
	                  this.fermer();
	               }
	            }
            }
         }
      }
   }
}